@@ -15,25 +15,37 @@
 #include <config.h>
 #endif
 
-#include "game.h"
+/* utility */
 #include "log.h"
+
+/* common */
+#include "game.h"
 #include "movement.h"
 #include "unit.h"
 #include "unitlist.h"
 
+/* aicore */
 #include "path_finding.h"
 #include "pf_tools.h"
 
+/* server/advisors */
+#include "advgoto.h"
+#include "autoexplorer.h"
+
+/* server */
 #include "hand_gen.h"
+#include "srv_log.h"
 #include "unithand.h"
 #include "unittools.h"
 
-#include "aidata.h"
+/* server/advisors */
+#include "advdata.h"
+
+/* ai */
 #include "aiguard.h"
-#include "aiexplorer.h"
-#include "ailog.h"
 #include "aitools.h"
 #include "aiunit.h"
+#include "defaultai.h"
 
 #include "aiferry.h"
 
@@ -43,7 +55,11 @@
 /* 
  * This one is used only by ferryboats in ai.passenger field 
  */
-#define FERRY_AVAILABLE   -1      /* Boat is looking for a passenger */
+#define FERRY_AVAILABLE    -1     /* Boat is looking for a passenger */
+#define FERRY_ABANDON_BOSS  0     /* Passenger is assigned for boat, but boat
+                                   * might take another passenger. Probably
+                                   * passenger already left the boat*/
+
 /* 
  * The below is used only by passengers in ai.ferryboat field 
  */ 
@@ -77,16 +93,25 @@ void aiferry_init_stats(struct player *pplayer)
   ai->stats.available_boats = 0;
  
   unit_list_iterate(pplayer->units, punit) {
-    if (is_sailing_unit(punit) && is_ground_units_transport(punit)) {
-      ai->stats.boats++;
-      if (punit->ai.passenger == FERRY_AVAILABLE) {
-	ai->stats.available_boats++;
-      }
+    struct unit_ai *unit_data = def_ai_unit_data(punit);
+
+    if (is_sailing_unit(punit)) {
+      unit_class_iterate(punitclass) {
+        if (punitclass->move_type == LAND_MOVING
+            && can_unit_type_transport(unit_type(punit), punitclass)) {
+          /* Can transport some land units. */
+          ai->stats.boats++;
+          if (unit_data->passenger == FERRY_AVAILABLE) {
+            ai->stats.available_boats++;
+          }
+          break;
+        }
+      } unit_class_iterate_end;
     }
-    if (punit->ai.ferryboat == FERRY_WANTED) {
+    if (unit_data->ferryboat == FERRY_WANTED) {
       UNIT_LOG(LOG_DEBUG, punit, "wants a boat.");
       ai->stats.passengers++;
-    }  
+    }
   } unit_list_iterate_end;
 }
 
@@ -99,44 +124,86 @@ static void aiferry_print_stats(struct player *pplayer)
   struct ai_data *ai = ai_data_get(pplayer);
   int n = 1;
 
-  freelog(LOGLEVEL_FERRY_STATS, "Boat stats for %s[%d]", 
-	  player_name(pplayer),
-	  player_number(pplayer));
-  freelog(LOGLEVEL_FERRY_STATS, "Registered: %d free out of total %d",
-	  ai->stats.available_boats, ai->stats.boats);
+  log_base(LOGLEVEL_FERRY_STATS, "Boat stats for %s[%d]",
+           player_name(pplayer), player_number(pplayer));
+  log_base(LOGLEVEL_FERRY_STATS, "Registered: %d free out of total %d",
+           ai->stats.available_boats, ai->stats.boats);
   unit_list_iterate(pplayer->units, punit) {
-    if (is_sailing_unit(punit) && is_ground_units_transport(punit)) {
-      freelog(LOGLEVEL_FERRY_STATS, "#%d. %s[%d], psngr=%d", 
-	      n,
-	      unit_rule_name(punit),
-	      punit->id,
-	      punit->ai.passenger);
-      n++;
+    if (is_sailing_unit(punit)) {
+        unit_class_iterate(punitclass) {
+          if (punitclass->move_type == LAND_MOVING
+              && can_unit_type_transport(unit_type(punit), punitclass)) {
+            /* Can transport some land units. */
+            log_base(LOGLEVEL_FERRY_STATS, "#%d. %s[%d], psngr=%d", n,
+                     unit_rule_name(punit), punit->id, punit->server.ai->passenger);
+            n++;
+            break;
+          }
+        } unit_class_iterate_end;
     }
   } unit_list_iterate_end;
 }
 #endif /* LOGLEVEL_FERRY_STATS */
 
+/**************************************************************************
+  Initialize new ferry when player gets it
+**************************************************************************/
+void aiferry_init_ferry(struct unit *ferry)
+{
+  if (is_sailing_unit(ferry)) {
+    struct unit_ai *unit_data = def_ai_unit_data(ferry);
+
+    unit_class_iterate(punitclass) {
+      if (punitclass->move_type == LAND_MOVING
+          && can_unit_type_transport(unit_type(ferry), punitclass)) {
+        /* Can transport some land units, so is consider ferry */
+        struct ai_data *ai = ai_data_get(unit_owner(ferry));
+
+        unit_data->passenger = FERRY_AVAILABLE;
+        ai->stats.boats++;
+        ai->stats.available_boats++;
+
+        break;
+      }
+    } unit_class_iterate_end;
+  }
+}
+
+
 /**************************************************************************
   Use on a unit which no longer needs a boat. 
 **************************************************************************/
 void aiferry_clear_boat(struct unit *punit)
 {
-  if (punit->ai.ferryboat == FERRY_WANTED) {
-    struct ai_data *ai = ai_data_get(unit_owner(punit));
+  struct unit_ai *unit_data = def_ai_unit_data(punit);
 
-    ai->stats.passengers--;
+  if (unit_data->ferryboat == FERRY_WANTED) {
+    struct player *pplayer = unit_owner(punit);
+
+    if (is_ai_data_phase_open(pplayer)) {
+      struct ai_data *ai = ai_data_get(unit_owner(punit));
+
+      ai->stats.passengers--;
+    }
   } else {
-    struct unit *ferry = game_find_unit_by_number(punit->ai.ferryboat);
-    
-    if (ferry && ferry->ai.passenger == punit->id) {
-      /* punit doesn't want us anymore */
-      ai_data_get(unit_owner(ferry))->stats.available_boats++;
-      ferry->ai.passenger = FERRY_AVAILABLE;
+    struct unit *ferry = game_unit_by_number(unit_data->ferryboat);
+
+    if (ferry) {
+      struct unit_ai *ferry_data = def_ai_unit_data(ferry);
+
+      if (ferry_data->passenger == punit->id) {
+        /* punit doesn't want us anymore */
+        struct player *pplayer = unit_owner(ferry);
+
+        if (is_ai_data_phase_open(pplayer)) {
+          ai_data_get(pplayer)->stats.available_boats++;
+        }
+        ferry_data->passenger = FERRY_AVAILABLE;
+      }
     }
   }
 
-  punit->ai.ferryboat = FERRY_NONE;
+  unit_data->ferryboat = FERRY_NONE;
 }
 
 /**************************************************************************
@@ -146,6 +213,7 @@ void aiferry_clear_boat(struct unit *punit)
 static void aiferry_request_boat(struct unit *punit)
 {
   struct ai_data *ai = ai_data_get(unit_owner(punit));
+  struct unit_ai *unit_data = def_ai_unit_data(punit);
 
   /* First clear the previous assignments (just in case). */
   aiferry_clear_boat(punit);
@@ -154,10 +222,10 @@ static void aiferry_request_boat(struct unit *punit)
   ai->stats.passengers++;
   UNIT_LOG(LOG_DEBUG, punit, "requests a boat (total passengers=%d).",
            ai->stats.passengers);
-  punit->ai.ferryboat = FERRY_WANTED;
+  unit_data->ferryboat = FERRY_WANTED;
 
   /* Lastly, wait for ferry. */
-  punit->ai.done = TRUE;
+  unit_data->done = TRUE;
 }
 
 /**************************************************************************
@@ -165,20 +233,22 @@ static void aiferry_request_boat(struct unit *punit)
 **************************************************************************/
 static void aiferry_psngr_meet_boat(struct unit *punit, struct unit *pferry)
 {
-  assert(unit_owner(punit) == unit_owner(pferry));
+  struct unit_ai *ferry_data = def_ai_unit_data(pferry);
+
+  fc_assert_ret(unit_owner(punit) == unit_owner(pferry));
 
   /* First delete the unit from the list of passengers and 
    * release its previous ferry */
   aiferry_clear_boat(punit);
 
   /* If ferry was available, update the stats */
-  if (pferry->ai.passenger == FERRY_AVAILABLE) {
+  if (ferry_data->passenger == FERRY_AVAILABLE) {
     ai_data_get(unit_owner(pferry))->stats.available_boats--;
   }
 
   /* Exchange the phone numbers */
-  punit->ai.ferryboat = pferry->id;
-  pferry->ai.passenger = punit->id;
+  def_ai_unit_data(punit)->ferryboat = pferry->id;
+  ferry_data->passenger = punit->id;
 }
 
 /**************************************************************************
@@ -186,9 +256,11 @@ static void aiferry_psngr_meet_boat(struct unit *punit, struct unit *pferry)
 **************************************************************************/
 static void aiferry_make_available(struct unit *pferry)
 {
-  if (pferry->ai.passenger != FERRY_AVAILABLE) {
+  struct unit_ai *ferry_data = def_ai_unit_data(pferry);
+
+  if (ferry_data->passenger != FERRY_AVAILABLE) {
     ai_data_get(unit_owner(pferry))->stats.available_boats++;
-    pferry->ai.passenger = FERRY_AVAILABLE;
+    ferry_data->passenger = FERRY_AVAILABLE;
   }
 }
 
@@ -208,15 +280,23 @@ static int aiferry_avail_boats(struct player *pplayer)
   int boats = 0;
 
   unit_list_iterate(pplayer->units, punit) {
-    if (is_sailing_unit(punit) && is_ground_units_transport(punit) 
-	&& punit->ai.passenger == FERRY_AVAILABLE) {
-      boats++;
+    if (is_sailing_unit(punit)
+        && punit->server.ai->passenger == FERRY_AVAILABLE) {
+      unit_class_iterate(punitclass) {
+        if (punitclass->move_type == LAND_MOVING
+            && can_unit_type_transport(unit_type(punit), punitclass)) {
+          /* Can transport some land units. */
+          boats++;
+          break;
+        }
+      } unit_class_iterate_end;
     }
   } unit_list_iterate_end;
 
   if (boats != ai->stats.available_boats) {
-    freelog(LOGLEVEL_FERRY_STATS, "Player[%d] in turn %d: boats miscounted.",
-	    player_number(pplayer), game.info.turn);
+    log_base(LOGLEVEL_FERRY_STATS,
+             "Player[%d] in turn %d: boats miscounted.",
+             player_number(pplayer), game.info.turn);
     aiferry_print_stats(pplayer);
   }
 #endif /* LOGLEVEL_FERRY_STATS */
@@ -247,15 +327,15 @@ static int combined_land_sea_move(const struct tile *src_tile,
 {
   int move_cost;
 
-  if (is_ocean(tgt_tile->terrain)) {
+  if (is_ocean_tile(tgt_tile)) {
     /* Any-to-Sea */
     move_cost = 0;
-  } else if (is_ocean(src_tile->terrain)) {
+  } else if (is_ocean_tile(src_tile)) {
     /* Sea-to-Land */
     move_cost = PF_IMPOSSIBLE_MC;
   } else {
     /* Land-to-Land */
-    move_cost = map_move_cost_ai(src_tile, tgt_tile);
+    move_cost = map_move_cost_ai(param->owner, src_tile, tgt_tile);
   }
 
   return move_cost;
@@ -268,7 +348,7 @@ static int combined_land_sea_move(const struct tile *src_tile,
 static int sea_move(const struct tile *ptile, enum known_type known,
                     const struct pf_parameter *param)
 {
-  if (is_ocean(ptile->terrain)) {
+  if (is_ocean_tile(ptile)) {
     /* Approximately TURN_FACTOR / average ferry move rate 
      * we can pass a better guess of the move rate through param->data
      * but we don't know which boat we will find out there */
@@ -291,21 +371,44 @@ bool is_boat_free(struct unit *boat, struct unit *punit, int cap)
    * - Only available boats or boats that are already dedicated to this unit
    *   are eligible.
    * - Only boats with enough remaining capacity are eligible.
-   * - Land moving units cannot transport others over sea
-   * - Units that require fuel are lose hitpoints are not eligible
+   * - Only units that can travel at sea are eligible.
+   * - Units that require fuel are lose hitpoints are not eligible.
    */
-  enum unit_move_type boat_move_type = unit_type(boat)->move_type;
+  struct unit_class *ferry_class = unit_class(boat);
+  struct unit_ai *boat_data = def_ai_unit_data(boat);
 
   return (can_unit_transport(boat, punit)
-	  && !unit_has_orders(boat)
-	  && unit_owner(boat) == unit_owner(punit)
-	  && (boat->ai.passenger == FERRY_AVAILABLE
-	      || boat->ai.passenger == punit->id)
-	  && (get_transporter_capacity(boat) 
-	      - get_transporter_occupancy(boat) >= cap)
-          && boat_move_type != LAND_MOVING
+          && !unit_has_orders(boat)
+          && unit_owner(boat) == unit_owner(punit)
+          && (boat_data->passenger == FERRY_AVAILABLE
+              || boat_data->passenger == punit->id)
+          && (get_transporter_capacity(boat) 
+              - get_transporter_occupancy(boat) >= cap)
+          && ferry_class->ai.sea_move != MOVE_NONE
           && !unit_type(boat)->fuel
-          && unit_class(boat)->hp_loss_pct <= 0);
+          && !is_losing_hp(boat));
+}
+
+/****************************************************************************
+  Check if unit is boss in ferry
+****************************************************************************/
+bool is_boss_of_boat(struct unit *punit)
+{
+  struct unit *ferry;
+
+  if (punit->transported_by == -1) {
+    /* Not even in boat */
+    return FALSE;
+  }
+
+  ferry = player_unit_by_number(unit_owner(punit), punit->transported_by);
+
+  if (ferry != NULL
+      && def_ai_unit_data(ferry)->passenger == punit->id) {
+    return TRUE;
+  }
+
+  return FALSE;
 }
 
 /****************************************************************************
@@ -322,11 +425,21 @@ int aiferry_find_boat(struct unit *punit, int cap, struct pf_path **path)
   int best_id = 0;
   struct pf_parameter param;
   struct pf_map *search_map;
-  int ferryboat = punit->ai.ferryboat; /*currently assigned ferry*/
 
-  assert(0 < ferryboat
-	 || FERRY_NONE == ferryboat
-	 || FERRY_WANTED == ferryboat);
+  /* currently assigned ferry */
+  int ferryboat = def_ai_unit_data(punit)->ferryboat;
+
+  /* We may end calling pf_destroy_path for *path if it's not NULL.
+   * Most likely you are passing garbage or path you don't want
+   * destroyed if this assertion fails.
+   * Don't try to be clever and pass 'fallback' path that will be returned
+   * if no path is found. Instead check for NULL return value and then
+   * use fallback path in calling function. */
+  fc_assert_ret_val(path == NULL || *path == NULL, 0);
+
+  fc_assert_ret_val(0 < ferryboat
+                    || FERRY_NONE == ferryboat
+                    || FERRY_WANTED == ferryboat, 0);
   UNIT_LOG(LOGLEVEL_FINDFERRY, punit, "asked aiferry_find_boat for a boat");
 
   if (aiferry_avail_boats(unit_owner(punit)) <= 0 
@@ -337,15 +450,17 @@ int aiferry_find_boat(struct unit *punit, int cap, struct pf_path **path)
   }
 
   pft_fill_unit_parameter(&param, punit);
-  param.turn_mode = TM_WORST_TIME;
   param.get_TB = no_fights_or_unknown;
   param.get_EC = sea_move;
   param.get_MC = combined_land_sea_move;
 
-  search_map = pf_create_map(&param);
+  search_map = pf_map_new(&param);
 
-  pf_iterator(search_map, pos) {
-    int radius = (is_ocean(pos.tile->terrain) ? 1 : 0);
+  pf_map_positions_iterate(search_map, pos, TRUE) {
+   /* Should this be !can_unit_exist_at_tile() instead of is_ocean() some day?
+    * That would allow special units to wade in shallow coast waters to meet
+    * ferry where deep sea starts. */
+    int radius = (is_ocean_tile(pos.tile) ? 1 : 0);
 
     if (pos.turn + pos.total_EC/PF_TURN_FACTOR > best_turns) {
       /* Won't find anything better */
@@ -374,9 +489,9 @@ int aiferry_find_boat(struct unit *punit, int cap, struct pf_path **path)
 		     aunit->moves_left);
 	    if (path) {
              if (*path) {
-                pf_destroy_path(*path);
+                pf_path_destroy(*path);
               }
-	      *path = pf_next_get_path(search_map);
+	      *path = pf_map_iter_path(search_map);
 	    }
             best_turns = turns;
             best_id = aunit->id;
@@ -384,8 +499,8 @@ int aiferry_find_boat(struct unit *punit, int cap, struct pf_path **path)
         }
       } unit_list_iterate_end;
     } square_iterate_end;
-  } pf_iterator_end;
-  pf_destroy_map(search_map);
+  } pf_map_positions_iterate_end;
+  pf_map_destroy(search_map);
 
   return best_id;
 }
@@ -420,8 +535,8 @@ static void ai_activate_passengers(struct unit *ferry)
 {
   unit_list_iterate_safe(ferry->tile->units, aunit) {
     if (aunit->transported_by == ferry->id) {
-      handle_unit_activity_request(aunit, ACTIVITY_IDLE);
-      aunit->ai.done = FALSE;
+      unit_activity_handling(aunit, ACTIVITY_IDLE);
+      def_ai_unit_data(aunit)->done = FALSE;
       ai_manage_unit(unit_owner(aunit), aunit);
     }
   } unit_list_iterate_safe_end;
@@ -441,12 +556,12 @@ bool ai_amphibious_goto_constrained(struct unit *ferry,
 {
   bool alive = TRUE;
   struct player *pplayer = unit_owner(passenger);
-  struct pf_map *map = NULL;
-  struct pf_path *path = NULL;
+  struct pf_map *pfm;
+  struct pf_path *path;
 
-  assert(pplayer->ai.control);
-  assert(!unit_has_orders(passenger));
-  assert(ferry->tile == passenger->tile);
+  fc_assert_ret_val(pplayer->ai_controlled, TRUE);
+  fc_assert_ret_val(!unit_has_orders(passenger), TRUE);
+  fc_assert_ret_val(ferry->tile == passenger->tile, TRUE);
 
   ptile = immediate_destination(passenger, ptile);
 
@@ -460,13 +575,13 @@ bool ai_amphibious_goto_constrained(struct unit *ferry,
     return TRUE;
   }
 
-  map = pf_create_map(&parameter->combined);
-  path = pf_get_path(map, ptile);
+  pfm = pf_map_new(&parameter->combined);
+  path = pf_map_path(pfm, ptile);
 
   if (path) {
     ai_log_path(passenger, path, &parameter->combined);
     /* Sea leg */
-    alive = ai_follow_path(ferry, path, ptile);
+    alive = adv_follow_path(ferry, path, ptile);
     if (alive && passenger->tile != ptile) {
       /* Ferry has stopped; it is at the landing beach or
        * has run out of movement points */
@@ -478,13 +593,13 @@ bool ai_amphibious_goto_constrained(struct unit *ferry,
         return alive;
       }
       next_tile = path->positions[1].tile;
-      if (!is_ocean(next_tile->terrain)) {
+      if (!is_ocean_tile(next_tile)) {
 	UNIT_LOG(LOG_DEBUG, passenger, "Our boat has arrived "
 		 "[%d](moves left: %d)", ferry->id, ferry->moves_left);
 	UNIT_LOG(LOG_DEBUG, passenger, "Disembarking to (%d,%d)",
 		 TILE_XY(next_tile));
 	/* Land leg */
-	alive = ai_follow_path(passenger, path, ptile);
+        alive = adv_follow_path(passenger, path, ptile);
 	if (0 < ferry->moves_left
             && (!alive || ferry->tile != passenger->tile)) {
 	  /* The passenger is no longer on the ferry,
@@ -506,8 +621,8 @@ bool ai_amphibious_goto_constrained(struct unit *ferry,
     UNIT_LOG(LOG_DEBUG, passenger, "no path to destination");
   }
 
-  pf_destroy_path(path);
-  pf_destroy_map(map);
+  pf_path_destroy(path);
+  pf_map_destroy(pfm);
 
   return alive;
 }
@@ -561,26 +676,26 @@ bool aiferry_gobyboat(struct player *pplayer, struct unit *punit,
 
     if (!is_ocean_near_tile(punit->tile)) {
       struct pf_path *path_to_ferry = NULL;
-      
+
       boatid = aiferry_find_boat(punit, 2, &path_to_ferry);
       if (boatid <= 0) {
-        UNIT_LOG(LOGLEVEL_GOBYBOAT, punit, 
-		 "in ai_gothere cannot find any boats.");
-        punit->ai.done = TRUE; /* Nothing to do */
+        UNIT_LOG(LOGLEVEL_GOBYBOAT, punit,
+                 "in ai_gothere cannot find any boats.");
+        def_ai_unit_data(punit)->done = TRUE; /* Nothing to do */
         return FALSE;
       }
 
-      ferryboat = game_find_unit_by_number(boatid);
+      ferryboat = game_unit_by_number(boatid);
       UNIT_LOG(LOGLEVEL_GOBYBOAT, punit, "found boat[%d](%d,%d), going there", 
 	       boatid, TILE_XY(ferryboat->tile));
       /* The path can be amphibious so we will stop at the coast.  
        * It might not lead _onto_ the boat. */
-      if (!ai_unit_execute_path(punit, path_to_ferry)) { 
+      if (!adv_unit_execute_path(punit, path_to_ferry)) { 
         /* Died. */
-	pf_destroy_path(path_to_ferry);
+	pf_path_destroy(path_to_ferry);
         return FALSE;
       }
-      pf_destroy_path(path_to_ferry);
+      pf_path_destroy(path_to_ferry);
     }
 
     if (!is_ocean_near_tile(punit->tile)) {
@@ -597,7 +712,7 @@ bool aiferry_gobyboat(struct player *pplayer, struct unit *punit,
     }
 
     /* Ok, a boat found, try boarding it */
-    ferryboat = game_find_unit_by_number(boatid);
+    ferryboat = game_unit_by_number(boatid);
     UNIT_LOG(LOGLEVEL_GOBYBOAT, punit, "found a nearby boat[%d](%d,%d)",
 	     ferryboat->id, TILE_XY(ferryboat->tile));
     /* Setting ferry now in hope it won't run away even 
@@ -618,12 +733,12 @@ bool aiferry_gobyboat(struct player *pplayer, struct unit *punit,
     }
 
     handle_unit_load(pplayer, punit->id, ferryboat->id);
-    assert(punit->transported_by > 0);
+    fc_assert(punit->transported_by > 0);
   }
 
   if (punit->transported_by > 0) {
     /* We are on a boat, ride it! */
-    struct unit *ferryboat = game_find_unit_by_number(punit->transported_by);
+    struct unit *ferryboat = game_unit_by_number(punit->transported_by);
 
     /* Check if we are the passenger-in-charge */
     if (is_boat_free(ferryboat, punit, 0)) {
@@ -646,7 +761,7 @@ bool aiferry_gobyboat(struct player *pplayer, struct unit *punit,
         } else if (bodyguard->moves_left <= 0) {
           /* Wait for me, I'm cooooming!! */
           UNIT_LOG(LOGLEVEL_GOBYBOAT, punit, "waiting for bodyguard");
-          punit->ai.done = TRUE;
+          def_ai_unit_data(punit)->done = TRUE;
           return FALSE;
         } else {
           /* Crap bodyguard. Got stuck somewhere. Ditch it! */
@@ -657,8 +772,8 @@ bool aiferry_gobyboat(struct player *pplayer, struct unit *punit,
         }
       }
       if (bodyguard) {
-        assert(same_pos(punit->tile, bodyguard->tile));
-	handle_unit_load(pplayer, bodyguard->id, ferryboat->id);
+        fc_assert(same_pos(punit->tile, bodyguard->tile));
+        handle_unit_load(pplayer, bodyguard->id, ferryboat->id);
       }
       if (!aiferry_goto_amphibious(ferryboat, punit, dest_tile)) {
         /* died */
@@ -666,17 +781,17 @@ bool aiferry_gobyboat(struct player *pplayer, struct unit *punit,
       }
       if (same_pos(punit->tile, dest_tile)) {
         /* Arrived */
-        handle_unit_activity_request(punit, ACTIVITY_IDLE);
+        unit_activity_handling(punit, ACTIVITY_IDLE);
       } else {
         /* We are in still transit */
-        punit->ai.done = TRUE;
+        def_ai_unit_data(punit)->done = TRUE;
         return FALSE;
       }
     } else {
       /* Waiting for the boss to load and move us */
       UNIT_LOG(LOGLEVEL_GOBYBOAT, punit, "Cannot command boat [%d],"
                " its boss is [%d]", 
-               ferryboat->id, ferryboat->ai.passenger);
+               ferryboat->id, def_ai_unit_data(ferryboat)->passenger);
       return FALSE;
     }
   }
@@ -697,7 +812,7 @@ bool aiferry_gobyboat(struct player *pplayer, struct unit *punit,
 static bool aiferry_findcargo(struct unit *pferry)
 {
   /* Path-finding stuff */
-  struct pf_map *map;
+  struct pf_map *pfm;
   struct pf_parameter parameter;
   int passengers = ai_data_get(unit_owner(pferry))->stats.passengers;
 
@@ -713,16 +828,14 @@ static bool aiferry_findcargo(struct unit *pferry)
    * might be "blocked" by unknown.  We don't want to fight though */
   parameter.get_TB = no_fights;
   
-  map = pf_create_map(&parameter);
-  while (pf_next(map)) {
-    struct pf_position pos;
+  pfm = pf_map_new(&parameter);
+  pf_map_tiles_iterate(pfm, ptile, TRUE) {
+    unit_list_iterate(ptile->units, aunit) {
+      struct unit_ai *unit_data = def_ai_unit_data(aunit);
 
-    pf_next_get_position(map, &pos);
-    
-    unit_list_iterate(pos.tile->units, aunit) {
       if (unit_owner(pferry) == unit_owner(aunit) 
-	  && (aunit->ai.ferryboat == FERRY_WANTED
-	      || aunit->ai.ferryboat == pferry->id)) {
+          && (unit_data->ferryboat == FERRY_WANTED
+              || unit_data->ferryboat == pferry->id)) {
         UNIT_LOG(LOGLEVEL_FERRY, pferry, 
                  "Found a potential cargo %s[%d](%d,%d), going there",
                  unit_rule_name(aunit),
@@ -731,23 +844,23 @@ static bool aiferry_findcargo(struct unit *pferry)
 	pferry->goto_tile = aunit->tile;
         /* Exchange phone numbers */
         aiferry_psngr_meet_boat(aunit, pferry);
-        pf_destroy_map(map);
+        pf_map_destroy(pfm);
         return TRUE;
       }
     } unit_list_iterate_end;
-  }
+  } pf_map_tiles_iterate_end;
 
   /* False positive can happen if we cannot find a route to the passenger
    * because of an internal sea or enemy blocking the route */
   UNIT_LOG(LOGLEVEL_FERRY, pferry,
            "AI Passengers counting reported false positive %d", passengers);
-  pf_destroy_map(map);
+  pf_map_destroy(pfm);
   return FALSE;
 }
 
 /****************************************************************************
   A helper for ai_manage_ferryboat.  Finds a city that wants a ferry.  It
-  might signal for the ferry using pcity->ai.choice.need_boat field or
+  might signal for the ferry using pcity->server.ai.choice.need_boat field or
   it might simply be building a ferry of it's own.
 
   The city found will be set as the goto destination.
@@ -760,7 +873,7 @@ static bool aiferry_findcargo(struct unit *pferry)
 static bool aiferry_find_interested_city(struct unit *pferry)
 {
   /* Path-finding stuff */
-  struct pf_map *map;
+  struct pf_map *pfm;
   struct pf_parameter parameter;
   /* Early termination condition */
   int turns_horizon = FC_INFINITY;
@@ -773,38 +886,32 @@ static bool aiferry_find_interested_city(struct unit *pferry)
   /* We are looking for our own cities, no need to look into the unknown */
   parameter.get_TB = no_fights_or_unknown;
   parameter.omniscience = FALSE;
-  
-  map = pf_create_map(&parameter);
-  parameter.turn_mode = TM_WORST_TIME;
+  pfm = pf_map_new(&parameter);
 
-  /* We want to consider the place we are currently in too, hence the 
-   * do-while loop */
-  do { 
-    struct pf_position pos;
+  pf_map_positions_iterate(pfm, pos, TRUE) {
     struct city *pcity;
 
-    pf_next_get_position(map, &pos);
     if (pos.turn >= turns_horizon) {
       /* Won't be able to find anything better than what we have */
       break;
     }
 
-    pcity = tile_get_city(pos.tile);
+    pcity = tile_city(pos.tile);
     
     if (pcity && city_owner(pcity) == unit_owner(pferry)
-        && (pcity->ai.choice.need_boat 
-            || (pcity->production.is_unit
-		&& utype_has_role(utype_by_number(pcity->production.value),
-				 L_FERRYBOAT)))) {
+        && (def_ai_city_data(pcity)->choice.need_boat 
+            || (VUT_UTYPE == pcity->production.kind
+		&& utype_has_role(pcity->production.value.utype,
+				  L_FERRYBOAT)))) {
       bool really_needed = TRUE;
-      int turns = city_turns_to_build(pcity, pcity->production, TRUE);
+      int turns = city_production_turns_to_build(pcity, TRUE);
 
       UNIT_LOG(LOGLEVEL_FERRY, pferry, "%s (%d, %d) looks promising...", 
                city_name(pcity), TILE_XY(pcity->tile));
 
-      if (pos.turn > turns && pcity->production.is_unit
-          && utype_has_role(utype_by_number(pcity->production.value),
-			   L_FERRYBOAT)) {
+      if (pos.turn > turns
+          && VUT_UTYPE == pcity->production.kind
+          && utype_has_role(pcity->production.value.utype, L_FERRYBOAT)) {
         UNIT_LOG(LOGLEVEL_FERRY, pferry, "%s is NOT suitable: "
                  "will finish building its own ferry too soon",
                  city_name(pcity));
@@ -839,9 +946,9 @@ static bool aiferry_find_interested_city(struct unit *pferry)
         needed = TRUE;
       }
     }
-  } while (pf_next(map));
+  } pf_map_positions_iterate_end;
 
-  pf_destroy_map(map);
+  pf_map_destroy(pfm);
   return needed;
 }
 
@@ -859,45 +966,49 @@ void ai_manage_ferryboat(struct player *pplayer, struct unit *punit)
 {
   struct city *pcity;
   int sanity = punit->id;
+  struct unit_ai *unit_data;
 
   CHECK_UNIT(punit);
 
   /* Try to recover hitpoints if we are in a city, before we do anything */
   if (punit->hp < unit_type(punit)->hp 
-      && (pcity = tile_get_city(punit->tile))) {
+      && (pcity = tile_city(punit->tile))) {
     UNIT_LOG(LOGLEVEL_FERRY, punit, "waiting in %s to recover hitpoints", 
              city_name(pcity));
-    punit->ai.done = TRUE;
+    def_ai_unit_data(punit)->done = TRUE;
     return;
   }
 
   /* Check if we are an empty barbarian boat and so not needed */
   if (is_barbarian(pplayer) && get_transporter_occupancy(punit) == 0) {
-    wipe_unit(punit);
+    wipe_unit(punit, FALSE, NULL);
     return;
   }
 
+  unit_data = def_ai_unit_data(punit);
+
   do {
     /* Do we have the passenger-in-charge on board? */
     struct tile *ptile = punit->tile;
 
-    if (punit->ai.passenger > 0) {
-      struct unit *psngr = game_find_unit_by_number(punit->ai.passenger);
-      
+    if (unit_data->passenger > 0) {
+      struct unit *psngr = game_unit_by_number(unit_data->passenger);
+
       /* If the passenger-in-charge is adjacent, we should wait for it to 
        * board.  We will pass control to it later. */
-      if (!psngr 
-	  || real_map_distance(punit->tile, psngr->tile) > 1) {
-	UNIT_LOG(LOGLEVEL_FERRY, punit, 
-		 "recorded passenger[%d] is not on board, checking for others",
-		 punit->ai.passenger);
-	punit->ai.passenger = 0;
+      if (!psngr
+          || real_map_distance(punit->tile, psngr->tile) > 1) {
+        UNIT_LOG(LOGLEVEL_FERRY, punit, 
+                 "recorded passenger[%d] is not on board, checking for "
+                 "others", unit_data->passenger);
+        unit_data->passenger = FERRY_ABANDON_BOSS;
       }
     }
 
-    if (punit->ai.passenger <= 0) {
+    if (unit_data->passenger == FERRY_AVAILABLE
+        || unit_data->passenger == FERRY_ABANDON_BOSS) {
       struct unit *candidate = NULL;
-    
+
       /* Try to select passanger-in-charge from among our passengers */
       unit_list_iterate(ptile->units, aunit) {
         if (unit_owner(aunit) != pplayer 
@@ -907,8 +1018,8 @@ void ai_manage_ferryboat(struct player *pplayer, struct unit *punit)
            * when we don't save these values in a savegame. */
           continue;
         }
-      
-        if (aunit->ai.ai_role != AIUNIT_ESCORT) {
+
+        if (aunit->server.adv->role != AIUNIT_ESCORT) {
           candidate = aunit;
           break;
         } else {
@@ -926,11 +1037,11 @@ void ai_manage_ferryboat(struct player *pplayer, struct unit *punit)
       }
     }
 
-    if (punit->ai.passenger > 0) {
-      int bossid = punit->ai.passenger;    /* Loop prevention */
-      struct unit *boss = game_find_unit_by_number(punit->ai.passenger);
+    if (unit_data->passenger > 0) {
+      int bossid = unit_data->passenger;    /* Loop prevention */
+      struct unit *boss = game_unit_by_number(unit_data->passenger);
 
-      assert(boss != NULL);
+      fc_assert_ret(NULL != boss);
 
       if (unit_has_type_flag(boss, F_SETTLERS) || unit_has_type_flag(boss, F_CITIES)) {
         /* Temporary hack: settlers all go in the end, forcing them 
@@ -943,20 +1054,20 @@ void ai_manage_ferryboat(struct player *pplayer, struct unit *punit)
 		boss->id);
       ai_manage_unit(pplayer, boss);
     
-      if (!game_find_unit_by_number(sanity) || punit->moves_left <= 0) {
+      if (!game_unit_by_number(sanity) || punit->moves_left <= 0) {
         return;
       }
-      if (game_find_unit_by_number(bossid)) {
+      if (game_unit_by_number(bossid)) {
 	if (same_pos(punit->tile, boss->tile)) {
 	  /* The boss decided to stay put on the ferry. We aren't moving. */
           UNIT_LOG(LOG_DEBUG, boss, "drove ferry - done for now");
-          boss->ai.done = TRUE;
-	  return;
-	} else if (get_transporter_occupancy(punit) != 0) {
-	  /* The boss isn't on the ferry, and we have other passengers?
-	   * Forget about him. */
-	  punit->ai.passenger = 0;
-	}
+          def_ai_unit_data(boss)->done = TRUE;
+          return;
+        } else if (get_transporter_occupancy(punit) != 0) {
+          /* The boss isn't on the ferry, and we have other passengers?
+           * Forget about him. */
+          unit_data->passenger = FERRY_ABANDON_BOSS;
+        }
       }
     } else {
       /* Cannot select a passenger-in-charge */
@@ -978,7 +1089,7 @@ void ai_manage_ferryboat(struct player *pplayer, struct unit *punit)
   UNIT_LOG(LOGLEVEL_FERRY, punit, "Ferryboat is not carrying anyone "
 	   "(moves left: %d).", punit->moves_left);
   aiferry_make_available(punit);
-  handle_unit_activity_request(punit, ACTIVITY_IDLE);
+  unit_activity_handling(punit, ACTIVITY_IDLE);
   ai_unit_new_role(punit, AIUNIT_NONE, NULL);
   CHECK_UNIT(punit);
 
@@ -989,10 +1100,10 @@ void ai_manage_ferryboat(struct player *pplayer, struct unit *punit)
     if (ai_unit_goto(punit, punit->goto_tile)) {
       if (is_tiles_adjacent(punit->tile, punit->goto_tile)
           || same_pos(punit->tile, punit->goto_tile)) {
-        struct unit *cargo = game_find_unit_by_number(punit->ai.passenger);
+        struct unit *cargo = game_unit_by_number(unit_data->passenger);
 
         /* See if passenger can jump on board! */
-        assert(cargo != punit);
+        fc_assert_ret(cargo != punit);
         ai_manage_unit(pplayer, cargo);
       }
     }
@@ -1003,20 +1114,20 @@ void ai_manage_ferryboat(struct player *pplayer, struct unit *punit)
   if (aiferry_find_interested_city(punit)) {
     if (same_pos(punit->tile, punit->goto_tile)) {
       UNIT_LOG(LOGLEVEL_FERRY, punit, "staying in city that needs us");
-      punit->ai.done = TRUE;
+      unit_data->done = TRUE;
       return;
     } else {
       UNIT_LOG(LOGLEVEL_FERRY, punit, "going to city that needs us");
       if (ai_unit_goto(punit, punit->goto_tile)
           && same_pos(punit->tile, punit->goto_tile)) {
-        punit->ai.done = TRUE; /* save some CPU */
+        unit_data->done = TRUE; /* save some CPU */
       }
       return;
     }
   }
 
   UNIT_LOG(LOGLEVEL_FERRY, punit, "Passing control of ferry to explorer code");
-  switch (ai_manage_explorer(punit)) {
+  switch (manage_auto_explorer(punit)) {
   case MR_DEATH:
     /* don't use punit! */
     return;
@@ -1024,7 +1135,7 @@ void ai_manage_ferryboat(struct player *pplayer, struct unit *punit)
     /* FIXME: continue moving? */
     break;
   default:
-    punit->ai.done = TRUE;
+    unit_data->done = TRUE;
     break;
   };
 
@@ -1033,7 +1144,7 @@ void ai_manage_ferryboat(struct player *pplayer, struct unit *punit)
     if (pcity) {
       punit->goto_tile = pcity->tile;
       UNIT_LOG(LOGLEVEL_FERRY, punit, "No work, going home");
-      punit->ai.done = TRUE;
+      unit_data->done = TRUE;
       ai_unit_new_role(punit, AIUNIT_NONE, NULL);
       (void) ai_unit_goto(punit, pcity->tile);
     }
