@@ -15,41 +15,47 @@
 #include <config.h>
 #endif
 
-#include <assert.h>
 #include <string.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
-/* common & utility */
-#include "city.h"
-#include "dataio.h"
-#include "events.h"
+/* utility */
 #include "fcintl.h"
-#include "game.h"
-#include "government.h"
-#include "hash.h"
 #include "log.h"
 #include "mem.h"
-#include "packets.h"
-#include "shared.h"		/* for MIN() */
+#include "shared.h"             /* for MIN() */
 #include "specialist.h"
 #include "support.h"
 #include "timing.h"
 
+/* common */
+#include "city.h"
+#include "dataio.h"
+#include "events.h"
+#include "game.h"
+#include "government.h"
+#include "packets.h"
+#include "specialist.h"
+
 /* client */
-#include "agents.h"
 #include "attribute.h"
+#include "client_main.h"
+#include "climisc.h"
+#include "packhand.h"
+
+/* client/include */
 #include "chatline_g.h"
 #include "citydlg_g.h"
 #include "cityrep_g.h"
-#include "client_main.h"
-#include "climisc.h"
 #include "messagewin_g.h"
-#include "packhand.h"
+
+/* client/agents */
+#include "agents.h"
 
 #include "cma_core.h"
 
+
 /*
  * The CMA is an agent. The CMA will subscribe itself to all city
  * events. So if a city changes the callback function city_changed is
@@ -62,10 +68,10 @@
  defines, structs, globals, forward declarations
 *****************************************************************************/
 
-#define APPLY_RESULT_LOG_LEVEL				LOG_DEBUG
-#define HANDLE_CITY_LOG_LEVEL				LOG_DEBUG
-#define HANDLE_CITY_LOG_LEVEL2				LOG_DEBUG
-#define RESULTS_ARE_EQUAL_LOG_LEVEL			LOG_DEBUG
+#define log_apply_result                log_debug
+#define log_handle_city                 log_debug
+#define log_handle_city2                log_debug
+#define log_results_are_equal           log_debug
 
 #define SHOW_TIME_STATS                                 FALSE
 #define SHOW_APPLY_RESULT_ON_SERVER_ERRORS              FALSE
@@ -86,12 +92,12 @@ static struct {
  Returns TRUE iff the two results are equal. Both results have to be
  results for the given city.
 *****************************************************************************/
-static bool my_results_are_equal(const struct cm_result *const result1,
-                                 const struct cm_result *const result2)
+static bool my_results_are_equal(const struct cm_result *result1,
+                                 const struct cm_result *result2)
 {
 #define T(x) if (result1->x != result2->x) { \
-	freelog(RESULTS_ARE_EQUAL_LOG_LEVEL, #x); \
-	return FALSE; }
+  log_results_are_equal(#x); \
+  return FALSE; }
 
   T(disorder);
   T(happy);
@@ -104,14 +110,16 @@ static bool my_results_are_equal(const struct cm_result *const result1,
     T(surplus[stat]);
   } output_type_iterate_end;
 
-  city_map_iterate(x, y) {
-    if (is_free_worked_cxy(x, y)) {
+  fc_assert_ret_val(result1->city_radius_sq == result2->city_radius_sq,
+                    FALSE);
+  city_map_iterate(result1->city_radius_sq, cindex, x, y) {
+    if (is_free_worked_index(cindex)) {
       continue;
     }
 
-    if (result1->worker_positions_used[x][y] !=
-	result2->worker_positions_used[x][y]) {
-      freelog(RESULTS_ARE_EQUAL_LOG_LEVEL, "worker_positions_used");
+    if (result1->worker_positions[cindex]
+        != result2->worker_positions[cindex]) {
+      log_results_are_equal("worker_positions");
       return FALSE;
     }
   } city_map_iterate_end;
@@ -128,7 +136,7 @@ static bool my_results_are_equal(const struct cm_result *const result1,
 *****************************************************************************/
 static bool check_city(int city_id, struct cm_parameter *parameter)
 {
-  struct city *pcity = game_find_city_by_number(city_id);
+  struct city *pcity = game_city_by_number(city_id);
   struct cm_parameter dummy;
 
   if (!parameter) {
@@ -153,17 +161,18 @@ static bool check_city(int city_id, struct cm_parameter *parameter)
  the actual data matches the calculated one.
 *****************************************************************************/
 static bool apply_result_on_server(struct city *pcity,
-				   const struct cm_result *const result)
+                                   const struct cm_result *result)
 {
   int first_request_id = 0, last_request_id = 0, i;
-  struct cm_result current_state;
+  int city_radius_sq = city_map_radius_sq_get(pcity);
+  struct cm_result *current_state = cm_result_new(pcity);;
   bool success;
   struct tile *pcenter = city_tile(pcity);
 
-  assert(result->found_a_valid);
-  cm_result_from_main_map(&current_state, pcity, TRUE);
+  fc_assert_ret_val(result->found_a_valid, FALSE);
+  cm_result_from_main_map(current_state, pcity);
 
-  if (my_results_are_equal(&current_state, result)
+  if (my_results_are_equal(current_state, result)
       && !ALWAYS_APPLY_AT_SERVER) {
     stats.apply_result_ignored++;
     return TRUE;
@@ -171,37 +180,34 @@ static bool apply_result_on_server(struct city *pcity,
 
   stats.apply_result_applied++;
 
-  freelog(APPLY_RESULT_LOG_LEVEL,
-          "apply_result_on_server(city %d=\"%s\")",
-          pcity->id,
-          city_name(pcity));
+  log_apply_result("apply_result_on_server(city %d=\"%s\")",
+                   pcity->id, city_name(pcity));
 
   connection_do_buffer(&client.conn);
 
   /* Do checks */
   if (pcity->size != cm_result_citizens(result)) {
-    freelog(LOG_ERROR,
-            "apply_result_on_server(city %d=\"%s\") bad result!",
-            pcity->id,
-            city_name(pcity));
+    log_error("apply_result_on_server(city %d=\"%s\") bad result!",
+              pcity->id, city_name(pcity));
     cm_print_city(pcity);
     cm_print_result(result);
     return FALSE;
   }
 
   /* Remove all surplus workers */
-  city_tile_iterate_skip_free_cxy(pcenter, ptile, x, y) {
+  city_tile_iterate_skip_free_worked(city_radius_sq, pcenter, ptile, index,
+                                     x, y) {
     if (tile_worked(ptile) == pcity
-     && !result->worker_positions_used[x][y]) {
-      freelog(APPLY_RESULT_LOG_LEVEL, "Removing worker at {%d,%d}.", x, y);
+        && !result->worker_positions[index]) {
+      log_apply_result("Removing worker at {%d,%d}.", x, y);
 
       last_request_id =
         dsend_packet_city_make_specialist(&client.conn, pcity->id, x, y);
       if (first_request_id == 0) {
-	first_request_id = last_request_id;
+        first_request_id = last_request_id;
       }
     }
-  } city_tile_iterate_skip_free_cxy_end;
+  } city_tile_iterate_skip_free_worked_end;
 
   /* Change the excess non-default specialists to default. */
   specialist_type_iterate(sp) {
@@ -210,8 +216,8 @@ static bool apply_result_on_server(struct city *pcity,
     }
 
     for (i = 0; i < pcity->specialists[sp] - result->specialists[sp]; i++) {
-      freelog(APPLY_RESULT_LOG_LEVEL, "Change specialist from %d to %d.",
-	      sp, DEFAULT_SPECIALIST);
+      log_apply_result("Change specialist from %d to %d.",
+                       sp, DEFAULT_SPECIALIST);
       last_request_id = city_change_specialist(pcity,
 					       sp, DEFAULT_SPECIALIST);
       if (first_request_id == 0) {
@@ -225,11 +231,12 @@ static bool apply_result_on_server(struct city *pcity,
   /* Set workers */
   /* FIXME: This code assumes that any toggled worker will turn into a
    * DEFAULT_SPECIALIST! */
-  city_tile_iterate_skip_free_cxy(pcenter, ptile, x, y) {
+  city_tile_iterate_skip_free_worked(city_radius_sq, pcenter, ptile, index,
+                                     x, y) {
     if (NULL == tile_worked(ptile)
-     && result->worker_positions_used[x][y]) {
-      freelog(APPLY_RESULT_LOG_LEVEL, "Putting worker at {%d,%d}.", x, y);
-      assert(city_can_work_tile(pcity, ptile));
+     && result->worker_positions[index]) {
+      log_apply_result("Putting worker at {%d,%d}.", x, y);
+      fc_assert_action(city_can_work_tile(pcity, ptile), break);
 
       last_request_id =
         dsend_packet_city_make_worker(&client.conn, pcity->id, x, y);
@@ -237,7 +244,7 @@ static bool apply_result_on_server(struct city *pcity,
 	first_request_id = last_request_id;
       }
     }
-  } city_tile_iterate_skip_free_cxy_end;
+  } city_tile_iterate_skip_free_worked_end;
 
   /* Set all specialists except DEFAULT_SPECIALIST (all the unchanged
    * ones remain as DEFAULT_SPECIALIST). */
@@ -247,8 +254,8 @@ static bool apply_result_on_server(struct city *pcity,
     }
 
     for (i = 0; i < result->specialists[sp] - pcity->specialists[sp]; i++) {
-      freelog(APPLY_RESULT_LOG_LEVEL, "Changing specialist from %d to %d.",
-	      DEFAULT_SPECIALIST, sp);
+      log_apply_result("Changing specialist from %d to %d.",
+                       DEFAULT_SPECIALIST, sp);
       last_request_id = city_change_specialist(pcity,
 					       DEFAULT_SPECIALIST, sp);
       if (first_request_id == 0) {
@@ -270,7 +277,6 @@ static bool apply_result_on_server(struct city *pcity,
 	dsend_packet_city_refresh(&client.conn, pcity->id);
     stats.refresh_forced++;
   }
-  reports_freeze_till(last_request_id);
 
   connection_do_unbuffer(&client.conn);
 
@@ -279,50 +285,42 @@ static bool apply_result_on_server(struct city *pcity,
 
     wait_for_requests("CMA", first_request_id, last_request_id);
     if (!check_city(city_id, NULL)) {
-      freelog(LOG_ERROR,
-              "apply_result_on_server(city %d=\"%s\") !check_city()!",
-              pcity->id,
-              city_name(pcity));
+      log_error("apply_result_on_server(city %d=\"%s\") !check_city()!",
+                pcity->id, city_name(pcity));
       return FALSE;
     }
   }
 
   /* Return. */
-  cm_result_from_main_map(&current_state, pcity, TRUE);
+  cm_result_from_main_map(current_state, pcity);
 
-  success = my_results_are_equal(&current_state, result);
+  success = my_results_are_equal(current_state, result);
   if (!success) {
     cm_clear_cache(pcity);
 
-    if (SHOW_APPLY_RESULT_ON_SERVER_ERRORS) {
-      freelog(LOG_ERROR,
-              "apply_result_on_server(city %d=\"%s\") no match!",
-              pcity->id,
-              city_name(pcity));
+#if SHOW_APPLY_RESULT_ON_SERVER_ERRORS
+      log_error("apply_result_on_server(city %d=\"%s\") no match!",
+                pcity->id, city_name(pcity));
 
-      freelog(LOG_TEST,
-              "apply_result_on_server(city %d=\"%s\") have:",
-              pcity->id,
-              city_name(pcity));
+      log_test("apply_result_on_server(city %d=\"%s\") have:",
+               pcity->id, city_name(pcity));
       cm_print_city(pcity);
-      cm_print_result(&current_state);
+      cm_print_result(current_state);
 
-      freelog(LOG_TEST,
-              "apply_result_on_server(city %d=\"%s\") want:",
-              pcity->id,
-              city_name(pcity));
+      log_test("apply_result_on_server(city %d=\"%s\") want:",
+               pcity->id, city_name(pcity));
       cm_print_result(result);
-    }
+#endif /* SHOW_APPLY_RESULT_ON_SERVER_ERRORS */
   }
 
-  freelog(APPLY_RESULT_LOG_LEVEL,
-          "apply_result_on_server() return %d.",
-          (int)success);
+  cm_result_destroy(current_state);
+
+  log_apply_result("apply_result_on_server() return %d.", (int) success);
   return success;
 }
 
 /****************************************************************************
- Prints the data of the stats struct via freelog(LOG_TEST,...).
+ Prints the data of the stats struct via log_test(...).
 *****************************************************************************/
 static void report_stats(void)
 {
@@ -332,12 +330,11 @@ static void report_stats(void)
   total = stats.apply_result_ignored + stats.apply_result_applied;
   per_mill = (stats.apply_result_ignored * 1000) / (total ? total : 1);
 
-  freelog(LOG_TEST,
-	  "CMA: apply_result: ignored=%2d.%d%% (%d) "
-	  "applied=%2d.%d%% (%d) total=%d",
-	  per_mill / 10, per_mill % 10, stats.apply_result_ignored,
-	  (1000 - per_mill) / 10, (1000 - per_mill) % 10,
-	  stats.apply_result_applied, total);
+  log_test("CMA: apply_result: ignored=%2d.%d%% (%d) "
+           "applied=%2d.%d%% (%d) total=%d",
+           per_mill / 10, per_mill % 10, stats.apply_result_ignored,
+           (1000 - per_mill) / 10, (1000 - per_mill) % 10,
+           stats.apply_result_applied, total);
 #endif
 }
 
@@ -360,37 +357,33 @@ static void release_city(int city_id)
 *****************************************************************************/
 static void handle_city(struct city *pcity)
 {
-  struct cm_result result;
+  struct cm_result *result = cm_result_new(pcity);
   bool handled;
   int i, city_id = pcity->id;
 
-  freelog(HANDLE_CITY_LOG_LEVEL,
-	  "handle_city(city %d=\"%s\") pos=(%d,%d) owner=%s",
-	  pcity->id,
-	  city_name(pcity),
-	  TILE_XY(pcity->tile),
-	  nation_rule_name(nation_of_city(pcity)));
+  log_handle_city("handle_city(city %d=\"%s\") pos=(%d,%d) owner=%s",
+                  pcity->id, city_name(pcity), TILE_XY(pcity->tile),
+                  nation_rule_name(nation_of_city(pcity)));
 
-  freelog(HANDLE_CITY_LOG_LEVEL2, "START handle city %d=\"%s\"",
-	  pcity->id,
-	  city_name(pcity));
+  log_handle_city2("START handle city %d=\"%s\"",
+                   pcity->id, city_name(pcity));
 
   handled = FALSE;
   for (i = 0; i < 5; i++) {
     struct cm_parameter parameter;
 
-    freelog(HANDLE_CITY_LOG_LEVEL2, "  try %d", i);
+    log_handle_city2("  try %d", i);
 
     if (!check_city(city_id, &parameter)) {
       handled = TRUE;	
       break;
     }
 
-    pcity = game_find_city_by_number(city_id);
+    pcity = game_city_by_number(city_id);
 
-    cm_query_result(pcity, &parameter, &result);
-    if (!result.found_a_valid) {
-      freelog(HANDLE_CITY_LOG_LEVEL2, "  no valid found result");
+    cm_query_result(pcity, &parameter, result);
+    if (!result->found_a_valid) {
+      log_handle_city2("  no valid found result");
 
       cma_release_city(pcity);
 
@@ -400,28 +393,30 @@ static void handle_city(struct city *pcity)
       handled = TRUE;
       break;
     } else {
-      if (!apply_result_on_server(pcity, &result)) {
-	freelog(HANDLE_CITY_LOG_LEVEL2, "  doesn't cleanly apply");
-	if (check_city(city_id, NULL) && i == 0) {
+      if (!apply_result_on_server(pcity, result)) {
+        log_handle_city2("  doesn't cleanly apply");
+        if (check_city(city_id, NULL) && i == 0) {
           create_event(city_tile(pcity), E_CITY_CMA_RELEASE, ftc_client,
                        _("The citizen governor has gotten confused dealing "
                          "with %s.  You may want to have a look."),
                        city_link(pcity));
-	}
+        }
       } else {
-	freelog(HANDLE_CITY_LOG_LEVEL2, "  ok");
-	/* Everything ok */
-	handled = TRUE;
-	break;
+        log_handle_city2("  ok");
+        /* Everything ok */
+        handled = TRUE;
+        break;
       }
     }
   }
 
-  pcity = game_find_city_by_number(city_id);
+  cm_result_destroy(result);
+
+  pcity = game_city_by_number(city_id);
 
   if (!handled) {
-    assert(pcity != NULL);
-    freelog(HANDLE_CITY_LOG_LEVEL2, "  not handled");
+    fc_assert_ret(pcity != NULL);
+    log_handle_city2("  not handled");
 
     create_event(city_tile(pcity), E_CITY_CMA_RELEASE, ftc_client,
                  _("The citizen governor has gotten confused dealing "
@@ -430,16 +425,13 @@ static void handle_city(struct city *pcity)
 
     cma_release_city(pcity);
 
-    freelog(LOG_ERROR,
-            "handle_city() CMA: %s has changed multiple times.",
-            city_name(pcity));
-    freelog(LOG_ERROR,
-            /* TRANS: No full stop after the URL, could cause confusion. */
-            _("Please report this message at %s"),
-            BUG_URL);
+    log_error("handle_city() CMA: %s has changed multiple times.",
+              city_name(pcity));
+    /* TRANS: No full stop after the URL, could cause confusion. */
+    log_error(_("Please report this message at %s"), BUG_URL);
   }
 
-  freelog(HANDLE_CITY_LOG_LEVEL2, "END handle city=(%d)", city_id);
+  log_handle_city2("END handle city=(%d)", city_id);
 }
 
 /****************************************************************************
@@ -447,7 +439,7 @@ static void handle_city(struct city *pcity)
 *****************************************************************************/
 static void city_changed(int city_id)
 {
-  struct city *pcity = game_find_city_by_number(city_id);
+  struct city *pcity = game_city_by_number(city_id);
 
   if (pcity) {
     cm_clear_cache(pcity);
@@ -480,10 +472,10 @@ void cma_init(void)
   struct agent self;
   struct timer *timer = stats.wall_timer;
 
-  freelog(LOG_DEBUG, "sizeof(struct cm_result)=%d",
-	  (unsigned int) sizeof(struct cm_result));
-  freelog(LOG_DEBUG, "sizeof(struct cm_parameter)=%d",
-	  (unsigned int) sizeof(struct cm_parameter));
+  log_debug("sizeof(struct cm_result)=%d",
+            (unsigned int) sizeof(struct cm_result));
+  log_debug("sizeof(struct cm_parameter)=%d",
+            (unsigned int) sizeof(struct cm_parameter));
 
   /* reset cache counters */
   memset(&stats, 0, sizeof(stats));
@@ -506,10 +498,9 @@ void cma_init(void)
 /****************************************************************************
 ...
 *****************************************************************************/
-bool cma_apply_result(struct city *pcity,
-		     const struct cm_result *const result)
+bool cma_apply_result(struct city *pcity, const struct cm_result *result)
 {
-  assert(!cma_is_city_under_agent(pcity, NULL));
+  fc_assert(!cma_is_city_under_agent(pcity, NULL));
   if (result->found_a_valid) {
     return apply_result_on_server(pcity, result);
   } else
@@ -522,17 +513,16 @@ bool cma_apply_result(struct city *pcity,
 void cma_put_city_under_agent(struct city *pcity,
 			      const struct cm_parameter *const parameter)
 {
-  freelog(LOG_DEBUG, "cma_put_city_under_agent(city %d=\"%s\")",
-	  pcity->id,
-	  city_name(pcity));
+  log_debug("cma_put_city_under_agent(city %d=\"%s\")",
+            pcity->id, city_name(pcity));
 
-  assert(city_owner(pcity) == client.conn.playing);
+  fc_assert_ret(city_owner(pcity) == client.conn.playing);
 
   cma_set_parameter(ATTR_CITY_CMA_PARAMETER, pcity->id, parameter);
 
   cause_a_city_changed_for_agent("CMA", pcity);
 
-  freelog(LOG_DEBUG, "cma_put_city_under_agent: return");
+  log_debug("cma_put_city_under_agent: return");
 }
 
 /****************************************************************************
@@ -585,12 +575,12 @@ bool cma_get_parameter(enum attr_city attr, int city_id,
   if (len == 0) {
     return FALSE;
   }
-  assert(len == SAVED_PARAMETER_SIZE);
+  fc_assert_ret_val(len == SAVED_PARAMETER_SIZE, FALSE);
 
   dio_input_init(&din, buffer, len);
 
   dio_get_uint8(&din, &version);
-  assert(version == 2);
+  fc_assert_ret_val(version == 2, FALSE);
 
   /* Initialize the parameter (includes some AI-only fields that aren't
    * touched below). */
@@ -633,7 +623,7 @@ void cma_set_parameter(enum attr_city attr, int city_id,
   dio_put_uint8(&dout, 0); /* Dummy value; used to be factor_target. */
   dio_put_bool8(&dout, parameter->require_happy);
 
-  assert(dio_output_used(&dout) == SAVED_PARAMETER_SIZE);
+  fc_assert(dio_output_used(&dout) == SAVED_PARAMETER_SIZE);
 
   attr_city_set(attr, city_id, SAVED_PARAMETER_SIZE, buffer);
 }
