@@ -21,8 +21,8 @@
 #include <windowsx.h>
 #include <commctrl.h>
 
+/* common & utility */
 #include "fcintl.h"
-#include "game.h"
 #include "log.h"
 #include "map.h"
 #include "mem.h"
@@ -31,10 +31,11 @@
 #include "support.h"
 #include "version.h"
 
+/* client */
 #include "chatline.h"
-#include "civclient.h"
+#include "client_main.h"
 #include "climisc.h"
-#include "clinet.h"
+#include "clinet.h"		/* connect_to_server() */
 #include "colors.h"
 #include "connectdlg_common.h"
 #include "connectdlg.h"
@@ -133,7 +134,7 @@ void handle_authentication_req(enum authentication_type type, char *message)
       struct packet_authentication_reply reply;
 
       sz_strlcpy(reply.password, password);
-      send_packet_authentication_reply(&aconnection, &reply);
+      send_packet_authentication_reply(&client.conn, &reply);
       return;
     } else {
       dialog_config = ENTER_PASSWORD_TYPE;
@@ -155,7 +156,7 @@ void handle_authentication_req(enum authentication_type type, char *message)
 /**************************************************************************
   this regenerates the player information from a loaded game on the server.
 **************************************************************************/
-void handle_game_load(struct packet_game_load *packet)
+void handle_game_load(bool load_successful, char *filename)
 { 
   char *row[3];
   int i;
@@ -167,14 +168,14 @@ void handle_game_load(struct packet_game_load *packet)
   }
 
   /* we couldn't load the savegame, we could have gotten the name wrong, etc */
-  if (!packet->load_successful) {
+  if (!load_successful) {
     SetWindowText(connect_dlg, _("Couldn't load the savegame"));
     return;
   } else {
-    char *buf = strrchr(packet->load_filename, '/');
+    char *buf = strrchr(load_filename, '/');
 
     if (buf == NULL) {
-      buf = packet->load_filename;
+      buf = load_filename;
     } else {
       buf++;
     }
@@ -184,12 +185,12 @@ void handle_game_load(struct packet_game_load *packet)
   ShowWindow(start_dlg, SW_HIDE);
   ShowWindow(players_dlg, SW_SHOWNORMAL);
 
-  game.info.nplayers = packet->nplayers;
+  set_player_count(packet->nplayers);
   ListView_DeleteAllItems(players_listview);
 
-  for (i = 0; i < packet->nplayers; i++) {
+  players_iterate(pplayer) {
     const char *nation_name;
-    struct nation_type *pnation = nation_by_number(packet->nations[i]);
+    struct nation_type *pnation = nation_of_player(pplayer);
 
     if (pnation == NO_NATION_SELECTED) {
       nation_name = "";
@@ -197,23 +198,15 @@ void handle_game_load(struct packet_game_load *packet)
       nation_name = nation_adjective_translation(pnation);
     }
 
-    row[0] = packet->name[i];
-    row[1] = (char *)nation_name;
-    row[2] = packet->is_alive[i] ? _("Alive") : _("Dead");
-    row[3] = packet->is_ai[i] ? _("AI") : _("Human");
+    row[0] = (char *) player_name(pplayer);
+    row[1] = (char *) nation_name;
+    row[2] = (char *) pplayer->is_alive ? _("Alive") : _("Dead");
+    row[3] = (char *) pplayer->ai_controlled ? _("AI") : _("Human");
     fcwin_listview_add_row(players_listview, 0, 4, row);
-  }
+  } players_iterate_end;
 
-  /* if nplayers is zero, we suppose it's a scenario */
-  if (packet->load_successful && packet->nplayers == 0) {
+  if (game.info.is_new_game) {
     send_chat("/take -");
-
-    /* create a false entry */
-    row[0] = user_name;
-    row[1] = "";
-    row[2] = _("Alive");
-    row[3] = _("Human");
-    fcwin_listview_add_row(players_listview, 0, 4, row);
   }
 
   ListView_SetColumnWidth(players_listview, 0, LVSCW_AUTOSIZE);
@@ -276,7 +269,7 @@ static void connect_callback()
     if (connect_to_server(user_name, server_host, server_port,
                           errbuf, sizeof(errbuf)) != -1) {
     } else {
-      append_output_window(errbuf);
+      output_window_append(ftc_client, errbuf);
     }
 
     break; 
@@ -294,7 +287,7 @@ static void connect_callback()
     if (strncmp(reply.password, password, MAX_LEN_NAME) == 0) {
       EnableWindow(GetDlgItem(network_dlg, ID_CONNECTDLG_CONNECT), FALSE);
       password[0] = '\0';
-      send_packet_authentication_reply(&aconnection, &reply);
+      send_packet_authentication_reply(&client.conn, &reply);
     } else { 
       SetFocus(GetDlgItem(network_tabs[0], ID_CONNECTDLG_NAME));
       SetWindowText(GetDlgItem(network_tabs[0], ID_CONNECTDLG_NAME), "");
@@ -307,7 +300,7 @@ static void connect_callback()
     EnableWindow(GetDlgItem(network_dlg, ID_CONNECTDLG_CONNECT), FALSE);
     Edit_GetText(GetDlgItem(network_tabs[0], ID_CONNECTDLG_NAME),
 		 reply.password, 512);
-    send_packet_authentication_reply(&aconnection, &reply);
+    send_packet_authentication_reply(&client.conn, &reply);
     break;
   default:
     assert(0);
@@ -507,8 +500,8 @@ static LONG CALLBACK playersdlg_proc(HWND hWnd, UINT message, WPARAM wParam,
 static void server_scan_error(struct server_scan *scan,
 			      const char *message)
 {
-  append_output_window(message);
-  freelog(LOG_NORMAL, "%s", message);
+  output_window_append(ftc_client, message);
+  log_normal("%s", message);
   switch (server_scan_get_type(scan)) {
   case SERVER_SCAN_LOCAL:
     server_scan_finish(lan);
@@ -743,7 +736,7 @@ void handle_save_load(const char *title, bool is_save)
       GetCurrentDirectory(MAX_PATH, saved_games_dirname);
       SetCurrentDirectory(dirname);
 
-      fc_current_filename = mystrdup(ofn.lpstrFile);
+      fc_current_filename = fc_strdup(ofn.lpstrFile);
 
       send_save_game(fc_current_filename);
     } else {
@@ -751,8 +744,6 @@ void handle_save_load(const char *title, bool is_save)
     }
   } else {
     if (GetOpenFileName(&ofn)) {
-      char cmd[MAX_LEN_MSG];
-
       if (fc_current_filename) {
 	free(fc_current_filename);
       }
@@ -760,10 +751,9 @@ void handle_save_load(const char *title, bool is_save)
       GetCurrentDirectory(MAX_PATH, saved_games_dirname);
       SetCurrentDirectory(dirname);
 
-      fc_current_filename = mystrdup(ofn.lpstrFile);
+      fc_current_filename = fc_strdup(ofn.lpstrFile);
 
-      my_snprintf(cmd, sizeof(cmd), "/load %s", ofn.lpstrFile);
-      send_chat(cmd);
+      send_chat_printf("/load %s", ofn.lpstrFile);
     } else {
       SetCurrentDirectory(dirname);
     }
@@ -781,15 +771,12 @@ static void load_game_callback()
 }
 
 /**************************************************************************
-
+  Send parameters for new game to server.
 **************************************************************************/
 static void set_new_game_params(HWND win)
 {
-  int aifill;
-  int aiskill;
-
   char buf[512];
-  char aifill_str[MAX_LEN_MSG - MAX_LEN_USERNAME + 1];
+  int aiskill;
 
   if (!is_server_running()) {
     client_start_server();
@@ -797,18 +784,15 @@ static void set_new_game_params(HWND win)
 
   aiskill = ComboBox_GetCurSel(GetDlgItem(newgame_dlg,
 					  ID_NEWGAMEDLG_AISKILL));
-
-  my_snprintf(buf, sizeof(buf), "/%s", skill_level_names[aiskill]);
-  send_chat(buf);
+  send_chat_printf("/%s", ai_level_cmd(aiskill));
 
 #if 0 
   send_chat("/set autotoggle 1");
 #endif
-  Edit_GetText(GetDlgItem(newgame_dlg, ID_NEWGAMEDLG_AIFILL), buf, 512);
-  aifill = atoi(buf);
 
-  my_snprintf(aifill_str, sizeof(aifill_str), "/set aifill %d", aifill);
-  send_chat(aifill_str);
+  Edit_GetText(GetDlgItem(newgame_dlg, ID_NEWGAMEDLG_AIFILL), buf, 512);
+
+  send_chat_printf("/set aifill %d", atoi(buf));
 
   really_close_connection_dialog();
 
@@ -837,7 +821,7 @@ static LONG CALLBACK new_game_proc(HWND win, UINT message,
 	  popup_settable_options_dialog();
 	  break;
         case ID_NEWGAMEDLG_NATIONS:
-	  popup_races_dialog(game.player_ptr);
+	  popup_races_dialog(client.conn.playing);
 	  break;
 	case ID_CANCEL:
 	  client_kill_server(TRUE);
@@ -929,6 +913,7 @@ void gui_server_connect()
   struct fcwin_box *players_vbox;
   struct fcwin_box *newgame_vbox;
   LV_COLUMN lvc;
+  enum ai_level level;
   
   titles[0] =_(titles_[0]);
   titles[1] =_(titles_[1]);
@@ -1023,11 +1008,16 @@ void gui_server_connect()
 
   fcwin_box_add_static(hbox, _("AI skill level:"), 0, SS_LEFT, TRUE, TRUE,
 		       5);
-  fcwin_box_add_combo(hbox, NUM_SKILL_LEVELS, ID_NEWGAMEDLG_AISKILL,
+
+  fcwin_box_add_combo(hbox, number_of_ai_levels(), ID_NEWGAMEDLG_AISKILL,
 		      CBS_DROPDOWN | WS_VSCROLL, TRUE, TRUE, 5);
-  for (i = 0; i < NUM_SKILL_LEVELS; i++) {
-    ComboBox_AddString(GetDlgItem(newgame_dlg, ID_NEWGAMEDLG_AISKILL), 
-		       _(skill_level_names[i]));
+  for (level = 0; level < AI_LEVEL_LAST; level++) {
+    if (is_settable_ai_level(level)) {
+      /* We insert translated ai_level_name here. This cannot be used as
+       * command for setting ai level (that would be ai_level_cmd(level) */
+      ComboBox_AddString(GetDlgItem(newgame_dlg, ID_NEWGAMEDLG_AISKILL),
+                         ai_level_name(level));
+    }
   }
   ComboBox_SetCurSel(GetDlgItem(newgame_dlg, ID_NEWGAMEDLG_AISKILL), 1);
 
@@ -1068,7 +1058,7 @@ void gui_server_connect()
 		     TRUE, TRUE, 10);
   fcwin_box_add_edit(vbox, server_host, 40, ID_CONNECTDLG_HOST, 0,
 		     TRUE, TRUE, 10);
-  my_snprintf(buf, sizeof(buf), "%d", server_port);
+  fc_snprintf(buf, sizeof(buf), "%d", server_port);
   fcwin_box_add_edit(vbox, buf, 8, ID_CONNECTDLG_PORT, 0, TRUE, TRUE, 15);
 
   fcwin_box_add_box(hbox, vbox, TRUE, TRUE, 5);
