@@ -15,7 +15,6 @@
 #include <config.h>
 #endif
 
-#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -32,20 +31,27 @@
 #include <X11/Xaw/AsciiText.h>  
 #include <X11/Xaw/Viewport.h>
 
+/* utility */
 #include "fcintl.h"
-#include "game.h"
-#include "government.h"
-#include "map.h"
+#include "log.h"
 #include "mem.h"
-#include "packets.h"
-#include "player.h"
 #include "shared.h"
 #include "support.h"
 
-#include "chatline.h"
-#include "climisc.h"
-#include "clinet.h"
+/* common */
 #include "diptreaty.h"
+#include "game.h"
+#include "government.h"
+#include "map.h"
+#include "packets.h"
+#include "player.h"
+
+/* client */
+#include "client_main.h"
+#include "climisc.h"
+
+/* gui-xaw */
+#include "chatline.h"
 #include "gui_main.h"
 #include "gui_stuff.h"
 #include "mapview.h"
@@ -92,7 +98,7 @@ struct Diplomacy_dialog {
 
   Widget dip_erase_clause_command;
   
-  char clauselist_strings[MAX_NUM_CLAUSES+1][64];
+  char clauselist_strings[MAX_NUM_CLAUSES+1][128];
   char *clauselist_strings_ptrs[MAX_NUM_CLAUSES+1];
 };
 
@@ -220,7 +226,7 @@ static void popup_diplomacy_dialog(int other_player_id)
 {
   struct Diplomacy_dialog *pdialog = find_diplomacy_dialog(other_player_id);
 
-  if (game.player_ptr->ai.control) {
+  if (NULL == client.conn.playing || client.conn.playing->ai_controlled) {
     return;			/* Don't show if we are AI controlled. */
   }
 
@@ -228,7 +234,7 @@ static void popup_diplomacy_dialog(int other_player_id)
     Position x, y;
     Dimension width, height;
 
-    pdialog = create_diplomacy_dialog(game.player_ptr,
+    pdialog = create_diplomacy_dialog(client.conn.playing,
 				      player_by_number(other_player_id));
     XtVaGetValues(toplevel, XtNwidth, &width, XtNheight, &height, NULL);
     XtTranslateCoords(toplevel, (Position) width / 10,
@@ -246,23 +252,30 @@ static void popup_diplomacy_dialog(int other_player_id)
 static int fill_diplomacy_tech_menu(Widget popupmenu, 
 				    struct player *plr0, struct player *plr1)
 {
-  int i, flag;
-  
-  for(i=A_FIRST, flag=0; i<game.control.num_tech_types; i++) {
-    if (get_invention(plr0, i) == TECH_KNOWN
-        && (get_invention(plr1, i) == TECH_UNKNOWN
-	    || get_invention(plr1, i) == TECH_REACHABLE)
-        && tech_is_available(plr1, i)) {
+  int flag = 0;
+
+  if (!game.info.trading_tech) {
+    /* Trading advances deactivated. */
+    return 0;
+  }
+
+  advance_index_iterate(A_FIRST, i) {
+    if (player_invention_state(plr0, i) == TECH_KNOWN
+        && player_invention_reachable(plr1, i, FALSE)
+        && (player_invention_state(plr1, i) == TECH_UNKNOWN
+	    || player_invention_state(plr1, i) == TECH_PREREQS_KNOWN)) {
       Widget entry=
-	XtVaCreateManagedWidget(advance_name_translation(i), smeBSBObjectClass, 
-				popupmenu, NULL);
+	XtVaCreateManagedWidget(advance_name_translation(advance_by_number(i)),
+				smeBSBObjectClass,
+				popupmenu,
+				NULL);
       XtAddCallback(entry, XtNcallback, diplomacy_dialog_tech_callback,
 			 INT_TO_XTPOINTER((player_number(plr0) << 24) |
 					 (player_number(plr1) << 16) |
 					 i));
       flag=1;
     }
-  }
+  } advance_index_iterate_end;
   return flag;
 }
 
@@ -277,6 +290,12 @@ static int fill_diplomacy_city_menu(Widget popupmenu,
 				    struct player *plr0, struct player *plr1)
 {
   int i = 0, j = 0, n = city_list_size(plr0->cities);
+
+  if (game.info.trading_city) {
+    /* Trading cities deactivated. */
+    return 0;
+  }
+
   struct city **city_list_ptrs;
   if (n>0) {
     city_list_ptrs = fc_malloc(sizeof(struct city*)*n);
@@ -313,6 +332,7 @@ static int fill_diplomacy_city_menu(Widget popupmenu,
 struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0, 
 						 struct player *plr1)
 {
+  char plr0_buf[4 * MAX_LEN_NAME], plr1_buf[4 * MAX_LEN_NAME];
   char buf[512], *pheadlinem;
   struct Diplomacy_dialog *pdialog;
   Dimension width, height, maxwidth;
@@ -353,7 +373,7 @@ struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
 					       pdialog->dip_main_form, 
 					       NULL);
   
-  my_snprintf(buf, sizeof(buf), _("The %s offerings"),
+  fc_snprintf(buf, sizeof(buf), _("The %s offerings"),
 	      nation_adjective_for_player(plr0));
   pdialog->dip_headline0=XtVaCreateManagedWidget("dipheadline0", 
 						 labelWidgetClass, 
@@ -361,7 +381,7 @@ struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
 						 XtNlabel, buf,
 						 NULL);   
 
-  my_snprintf(buf, sizeof(buf), _("The %s offerings"),
+  fc_snprintf(buf, sizeof(buf), _("The %s offerings"),
 	      nation_adjective_for_player(plr1));
   pdialog->dip_headline1=XtVaCreateManagedWidget("dipheadline1", 
 						 labelWidgetClass, 
@@ -464,31 +484,33 @@ struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
 		 fill_diplomacy_city_menu(popupmenu, plr1, plr0));  
   
   /* End of trade city code */
-  
-  pdialog->dip_gold_input0=XtVaCreateManagedWidget("dipgoldinput0", 
-						   asciiTextWidgetClass,
-						   pdialog->dip_form0,
-						   NULL);
 
-  pdialog->dip_gold_input1=XtVaCreateManagedWidget("dipgoldinput1", 
-						   asciiTextWidgetClass,
-						   pdialog->dip_form1,
-						   NULL);
-  
-  my_snprintf(buf, sizeof(buf), _("Gold(max %d)"), plr0->economic.gold);
-  pdialog->dip_gold_label0=XtVaCreateManagedWidget("dipgoldlabel0", 
-						   labelWidgetClass,
-						   pdialog->dip_form0,
-						   XtNlabel, buf,
-						   NULL);
+  /* Trading gold */
+  if (game.info.trading_gold) {
+    pdialog->dip_gold_input0 = XtVaCreateManagedWidget("dipgoldinput0",
+                                                       asciiTextWidgetClass,
+                                                       pdialog->dip_form0, NULL);
+    fc_snprintf(buf, sizeof(buf), _("Gold(max %d)"), plr0->economic.gold);
+    pdialog->dip_gold_label0 = XtVaCreateManagedWidget("dipgoldlabel0",
+                                                       labelWidgetClass,
+                                                       pdialog->dip_form0,
+                                                       XtNlabel, buf, NULL);
 
-  my_snprintf(buf, sizeof(buf), _("Gold(max %d)"), plr1->economic.gold);
-  pdialog->dip_gold_label1=XtVaCreateManagedWidget("dipgoldlabel1", 
-						   labelWidgetClass,
-						   pdialog->dip_form1,
-						   XtNlabel, buf,
-						   NULL);
 
+    pdialog->dip_gold_input1 = XtVaCreateManagedWidget("dipgoldinput1",
+                                                       asciiTextWidgetClass,
+                                                       pdialog->dip_form1, NULL);
+    fc_snprintf(buf, sizeof(buf), _("Gold(max %d)"), plr1->economic.gold);
+    pdialog->dip_gold_label1 = XtVaCreateManagedWidget("dipgoldlabel1",
+                                                       labelWidgetClass,
+                                                       pdialog->dip_form1,
+                                                       XtNlabel, buf, NULL);
+  } else {
+    pdialog->dip_gold_input0 = NULL;
+    pdialog->dip_gold_input1 = NULL;
+    pdialog->dip_gold_label0 = NULL;
+    pdialog->dip_gold_label1 = NULL;
+  }
 
   pdialog->dip_vision_button0 =
     I_L(XtVaCreateManagedWidget("dipvisionbutton0",
@@ -533,18 +555,18 @@ struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
 				smeBSBObjectClass, popupmenu, NULL);
   XtAddCallback(entry, XtNcallback, diplomacy_dialog_alliance_callback,
 		(XtPointer)pdialog);
-  
-  my_snprintf(buf, sizeof(buf),
-	      _("This Eternal Treaty\n"
-		 "marks the results of the diplomatic work between\n"
-		 "The %s %s %s\nand\nThe %s %s %s"),
-	  nation_adjective_for_player(plr0),
-	  ruler_title_translation(plr0),
-	  player_name(plr0),
-	  nation_adjective_for_player(plr1),
-	  ruler_title_translation(plr1),
-	  player_name(plr1));
-  
+
+  fc_snprintf(buf, sizeof(buf),
+              /* TRANS: The <nation adjective> <ruler-title + player-name>
+               * E.g. "The Czech President Vaclav Havel". */
+              _("This Eternal Treaty\n"
+                "marks the results of the diplomatic work between\n"
+                "The %s %s\nand\nThe %s %s"),
+              nation_adjective_for_player(plr0),
+              ruler_title_for_player(plr0, plr0_buf, sizeof(plr0_buf)),
+              nation_adjective_for_player(plr1),
+              ruler_title_for_player(plr1, plr1_buf, sizeof(plr1_buf)));
+
   pheadlinem=create_centered_string(buf);
   pdialog->dip_headline1=XtVaCreateManagedWidget("dipheadlinem", 
 						 labelWidgetClass, 
@@ -575,7 +597,7 @@ struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
   XtVaSetValues(pdialog->dip_view, XtNwidth, width, NULL); 
   XtVaSetValues(pdialog->dip_clauselist, XtNwidth, width, NULL); 
 
-  my_snprintf(buf, sizeof(buf), _("%s view:"),
+  fc_snprintf(buf, sizeof(buf), _("%s view:"),
               nation_adjective_for_player(plr0));
   pdialog->dip_acceptlabel0=XtVaCreateManagedWidget("dipacceptlabel0",
 						    labelWidgetClass, 
@@ -587,7 +609,7 @@ struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
 						    pdialog->dip_formm, 
 						    XtNbitmap, get_thumb_pixmap(0),
 						    NULL);
-  my_snprintf(buf, sizeof(buf), _("%s view:"),
+  fc_snprintf(buf, sizeof(buf), _("%s view:"),
               nation_adjective_for_player(plr1));
   pdialog->dip_acceptlabel1=XtVaCreateManagedWidget("dipacceptlabel1",
 						    labelWidgetClass, 
@@ -633,13 +655,17 @@ struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
   XtVaGetValues(pdialog->dip_tech_menubutton0, XtNwidth, &width, NULL);
   XtVaGetValues(pdialog->dip_city_menubutton0, XtNwidth, &width, NULL);
   maxwidth=MAX(width, maxwidth);
-  XtVaGetValues(pdialog->dip_gold_input0, XtNwidth, &width, NULL);
+  if (pdialog->dip_gold_input0) {
+    XtVaGetValues(pdialog->dip_gold_input0, XtNwidth, &width, NULL);
+  }
   maxwidth=MAX(width, maxwidth);
   XtVaSetValues(pdialog->dip_map_menubutton0, XtNwidth, maxwidth, NULL);
   XtVaSetValues(pdialog->dip_tech_menubutton0, XtNwidth, maxwidth, NULL);
   XtVaSetValues(pdialog->dip_city_menubutton0, XtNwidth, maxwidth, NULL);
-  XtVaSetValues(pdialog->dip_gold_input0,  XtNwidth, maxwidth, NULL);
-  
+  if (pdialog->dip_gold_input0) {
+    XtVaSetValues(pdialog->dip_gold_input0,  XtNwidth, maxwidth, NULL);
+  }
+
   XtVaGetValues(pdialog->dip_formm, XtNheight, &height, NULL);
   XtVaSetValues(pdialog->dip_form0, XtNheight, height, NULL); 
   XtVaSetValues(pdialog->dip_form1, XtNheight, height, NULL); 
@@ -692,13 +718,13 @@ void diplomacy_dialog_tech_callback(Widget w, XtPointer client_data,
   int giver = (choice >> 24) & 0xff, dest = (choice >> 16) & 0xff, other;
   int tech = choice & 0xffff;
 
-  if (giver == game.info.player_idx) {
+  if (player_by_number(giver) == client.conn.playing) {
     other = dest;
   } else {
     other = giver;
   }
 
-  dsend_packet_diplomacy_create_clause_req(&aconnection, other, giver,
+  dsend_packet_diplomacy_create_clause_req(&client.conn, other, giver,
 					   CLAUSE_ADVANCE, tech);
 }
 
@@ -713,13 +739,13 @@ void diplomacy_dialog_city_callback(Widget w, XtPointer client_data,
   int giver = (choice >> 24) & 0xff, dest = (choice >> 16) & 0xff, other;
   int city = choice & 0xffff;
 
-  if (giver == game.info.player_idx) {
+  if (player_by_number(giver) == client.conn.playing) {
     other = dest;
   } else {
     other = giver;
   }
 
-  dsend_packet_diplomacy_create_clause_req(&aconnection, other, giver,
+  dsend_packet_diplomacy_create_clause_req(&client.conn, other, giver,
 					   CLAUSE_CITY, city);
 }
 
@@ -737,7 +763,7 @@ void diplomacy_dialog_erase_clause_callback(Widget w, XtPointer client_data,
 
     clause_list_iterate(pdialog->treaty.clauses, pclause) {
       if (i == ret->list_index) {
-	dsend_packet_diplomacy_remove_clause_req(&aconnection,
+	dsend_packet_diplomacy_remove_clause_req(&client.conn,
 						 player_number(pdialog->treaty.plr1),
 						 player_number(pclause->from),
 						 pclause->type,
@@ -761,7 +787,7 @@ void diplomacy_dialog_map_callback(Widget w, XtPointer client_data,
        pdialog->dip_map_menubutton0) ? pdialog->treaty.plr0 : pdialog->
       treaty.plr1;
 
-  dsend_packet_diplomacy_create_clause_req(&aconnection,
+  dsend_packet_diplomacy_create_clause_req(&client.conn,
 					   player_number(pdialog->treaty.plr1),
 					   player_number(pgiver), CLAUSE_MAP, 0);
 }
@@ -778,7 +804,7 @@ void diplomacy_dialog_seamap_callback(Widget w, XtPointer client_data,
        pdialog->dip_map_menubutton0) ? pdialog->treaty.plr0 : pdialog->
       treaty.plr1;
 
-  dsend_packet_diplomacy_create_clause_req(&aconnection,
+  dsend_packet_diplomacy_create_clause_req(&client.conn,
 					   player_number(pdialog->treaty.plr1),
 					   player_number(pgiver), CLAUSE_SEAMAP,
 					   0);
@@ -794,7 +820,7 @@ void diplomacy_dialog_vision_callback(Widget w, XtPointer client_data,
   struct player *pgiver = (w == pdialog->dip_vision_button0) ?
       pdialog->treaty.plr0 : pdialog->treaty.plr1;
 
-  dsend_packet_diplomacy_create_clause_req(&aconnection,
+  dsend_packet_diplomacy_create_clause_req(&client.conn,
 					   player_number(pdialog->treaty.plr1),
 					   player_number(pgiver), CLAUSE_VISION,
 					   0);
@@ -810,7 +836,7 @@ static void diplomacy_dialog_add_pact_clause(Widget w, XtPointer client_data,
   struct player *pgiver = (w == pdialog->dip_vision_button0) ?
       pdialog->treaty.plr0 : pdialog->treaty.plr1;
 
-  dsend_packet_diplomacy_create_clause_req(&aconnection,
+  dsend_packet_diplomacy_create_clause_req(&client.conn,
 					   player_number(pdialog->treaty.plr1),
 					   player_number(pgiver), type, 0);
 }
@@ -857,7 +883,7 @@ void diplomacy_dialog_close_callback(Widget w, XtPointer client_data,
 {
   struct Diplomacy_dialog *pdialog = (struct Diplomacy_dialog *) client_data;
 
-  dsend_packet_diplomacy_cancel_meeting_req(&aconnection,
+  dsend_packet_diplomacy_cancel_meeting_req(&client.conn,
 					    player_number(pdialog->treaty.plr1));
   close_diplomacy_dialog(pdialog);
 }
@@ -871,7 +897,7 @@ void diplomacy_dialog_accept_callback(Widget w, XtPointer client_data,
 {
   struct Diplomacy_dialog *pdialog = (struct Diplomacy_dialog *) client_data;
 
-  dsend_packet_diplomacy_accept_treaty_req(&aconnection,
+  dsend_packet_diplomacy_accept_treaty_req(&client.conn,
 					   player_number(pdialog->treaty.plr1));
 }
 
@@ -883,7 +909,7 @@ void close_diplomacy_dialog(struct Diplomacy_dialog *pdialog)
 {
   XtDestroyWidget(pdialog->dip_dialog_shell);
   
-  dialog_list_unlink(dialog_list, pdialog);
+  dialog_list_remove(dialog_list, pdialog);
   free(pdialog);
 }
 
@@ -892,7 +918,8 @@ void close_diplomacy_dialog(struct Diplomacy_dialog *pdialog)
 *****************************************************************/
 static struct Diplomacy_dialog *find_diplomacy_dialog(int other_player_id)
 {
-  struct player *plr0 = game.player_ptr, *plr1 = player_by_number(other_player_id);
+  struct player *plr0 = client.conn.playing;
+  struct player *plr1 = player_by_number(other_player_id);
 
   if (!dialog_list_list_has_been_initialised) {
     dialog_list = dialog_list_new();
@@ -937,16 +964,17 @@ void diplodlg_key_gold(Widget w)
     int amount;
     
     XtVaGetValues(w, XtNstring, &dp, NULL);
-    if (sscanf(dp, "%d", &amount) == 1 && amount >= 0
+    if (str_to_int(dp, &amount) && 0 <= amount
 	&& amount <= pgiver->economic.gold) {
-      dsend_packet_diplomacy_create_clause_req(&aconnection,
+      dsend_packet_diplomacy_create_clause_req(&client.conn,
 					       player_number(pdialog->treaty.plr1),
 					       player_number(pgiver),
 					       CLAUSE_GOLD, amount);
       XtVaSetValues(w, XtNstring, "", NULL);
+    } else {
+      output_window_append(ftc_client,
+                           _("Invalid amount of gold specified."));
     }
-    else
-      append_output_window(_("Invalid amount of gold specified."));
   }
 }
 
