@@ -121,9 +121,7 @@ static char *topic_list[1024];
 *****************************************************************/
 static void set_title_topic(const struct help_item *pitem)
 {
-  if (strcmp(pitem->topic, "Freeciv") == 0
-      || strcmp(pitem->topic, "About") == 0
-      || strcmp(pitem->topic, _("About")) == 0) {
+  if (strcmp(pitem->topic, _(HELP_ABOUT_ITEM)) == 0) {
     xaw_set_label(help_title, freeciv_name_version());
   } else {
     xaw_set_label(help_title, pitem->topic);
@@ -1196,7 +1194,7 @@ static void help_update_dialog(const struct help_item *pitem)
     help_update_government(pitem, top);
     break;
   case HELP_NATIONS:
-    help_update_nation(pitem, top, nation_by_rule_name(top));
+    help_update_nation(pitem, top, nation_by_translated_name(top));
     break;
   case HELP_TEXT:
   default:
