@@ -45,9 +45,11 @@
 #include "requirements.h"
 #include "specialist.h"
 #include "unit.h"
+#include "version.h"
 
 /* client */
 #include "client_main.h"
+#include "gui_main_g.h" /* client_string */
 
 #include "helpdata.h"
 
@@ -123,11 +125,9 @@ void free_help_texts(void)
 }
 
 /****************************************************************************
-  Insert generated data for the helpdate name.
-
-  Currently only for terrain ("TerrainAlterations") is such a table created.
+  Insert generated text for the helpdata "name".
 ****************************************************************************/
-static void insert_generated_table(char *outbuf, size_t outlen, const char *name)
+static void insert_generated_text(char *outbuf, size_t outlen, const char *name)
 {
   if (0 == strcmp (name, "TerrainAlterations")) {
     int rail_time = -1, clean_pollution_time = -1, clean_fallout_time = -1;
@@ -243,6 +243,13 @@ static void insert_generated_table(char *outbuf, size_t outlen, const char *name
         }
       } base_type_iterate_end;
     }
+  } else if (0 == strcmp (name, "FreecivVersion")) {
+    const char *ver = freeciv_name_version();
+    cat_snprintf(outbuf, outlen,
+                 /* TRANS: First %s is version string, e.g.,
+                  * "Freeciv version 2.3.0-beta1 (beta version)" (translated).
+                  * Second %s is client_string, e.g., "gui-gtk-2.0". */
+                 _("This is %s, %s client."), ver, client_string);
   }
   return;
 }
@@ -906,7 +913,7 @@ void boot_help_texts(struct player *pplayer)
             nations_iterate(pnation) {
               pitem = new_help_item(current_type);
               fc_snprintf(name, sizeof(name), "%*s%s", level, "",
-                          nation_rule_name(pnation));
+                          _(nation_rule_name(pnation)));
               pitem->topic = fc_strdup(name);
               pitem->text = fc_strdup("");
               help_list_append(category_nodes, pitem);
@@ -937,7 +944,7 @@ void boot_help_texts(struct player *pplayer)
       for (i=0; i<npara; i++) {
         const char *para = paras[i];
         if(strncmp(para, "$", 1)==0) {
-          insert_generated_table(long_buffer, sizeof(long_buffer), para+1);
+          insert_generated_text(long_buffer, sizeof(long_buffer), para+1);
         } else {
           sz_strlcat(long_buffer, _(para));
         }
@@ -1379,7 +1386,7 @@ char *helptext_unit(char *buf, size_t bufsz, struct player *pplayer,
                  _("* Can establish trade routes (must travel to target"
                    " city and must be at least %d tiles [in Manhattan"
                    " distance] from this unit's home city).\n"),
-                 game.info.trademindist);
+                 game.info.trademindist_new);
   }
   if (utype_has_flag(utype, F_HELP_WONDER)) {
     cat_snprintf(buf, bufsz,
@@ -1570,8 +1577,6 @@ char *helptext_unit(char *buf, size_t bufsz, struct player *pplayer,
         break;
       case LAND_MOVING:
         if (utype_has_flag(utype, F_DIPLOMAT)||utype_has_flag(utype, F_SPY)) {
-          CATLSTR(buf, bufsz,
-                  _("* Will be built as a veteran under communist governments.\n"));
           if (veteran_through_combat)
             CATLSTR(buf, bufsz,
                     _("* May be promoted after a successful mission.\n"));
@@ -2122,6 +2127,8 @@ void helptext_government(char *buf, size_t bufsz, struct player *pplayer,
 
       switch (peffect->type) {
       case EFT_UNHAPPY_FACTOR:
+        /* FIXME: EFT_MAKE_CONTENT_MIL_PER would cancel this out. We assume
+         * no-one will set both, so we don't bother handling it. */
         cat_snprintf(buf, bufsz,
                      PL_("* Military units away from home and field units"
                          " will each cause %d citizen to become unhappy.\n",
@@ -2130,6 +2137,15 @@ void helptext_government(char *buf, size_t bufsz, struct player *pplayer,
                          peffect->value),
                      peffect->value);
         break;
+      case EFT_MAKE_CONTENT_MIL:
+        cat_snprintf(buf, bufsz,
+                     PL_("* Each of your cities will avoid %d unhappiness"
+                         " caused by units.\n",
+                         "* Each of your cities will avoid %d unhappiness"
+                         " caused by units.\n",
+                         peffect->value),
+                     peffect->value);
+        break;
       case EFT_MAKE_CONTENT:
         cat_snprintf(buf, bufsz,
                      PL_("* Each of your cities will avoid %d unhappiness,"
@@ -2162,7 +2178,7 @@ void helptext_government(char *buf, size_t bufsz, struct player *pplayer,
           cat_snprintf(buf, bufsz,
                        /* TRANS: %s is the output type, like 'shield' or 'gold'. */
                        _("* You pay no %s upkeep for your units.\n"),
-                       astr_str(&outputs_and));
+                       astr_str(&outputs_or));
         } else if (peffect->value == 0) {
           CATLSTR(buf, bufsz,
                   _("* You pay no upkeep for your units.\n"));
