@@ -47,8 +47,16 @@ bool client_handle_packet(enum packet_type type, const void *packet)
     handle_server_shutdown();
     return TRUE;
 
-  case PACKET_ENDGAME_REPORT:
-    handle_endgame_report(packet);
+  case PACKET_ENDGAME_REPORT_OLD:
+    handle_endgame_report_old(packet);
+    return TRUE;
+
+  case PACKET_ENDGAME_PLAYER:
+    handle_endgame_player(packet);
+    return TRUE;
+
+  case PACKET_ENDGAME_REPORT_NEW:
+    handle_endgame_report_new(packet);
     return TRUE;
 
   case PACKET_TILE_INFO:
@@ -284,6 +292,10 @@ bool client_handle_packet(enum packet_type type, const void *packet)
     handle_ruleset_terrain_control(packet);
     return TRUE;
 
+  case PACKET_RULESETS_READY:
+    handle_rulesets_ready();
+    return TRUE;
+
   case PACKET_RULESET_NATION_GROUPS:
     handle_ruleset_nation_groups(packet);
     return TRUE;
