@@ -29,13 +29,17 @@
   Includes cmdlevel stuff, which is connection-based.
 ***************************************************************************/
 
-#include "shared.h"		/* MAX_LEN_ADDR, bool type */
+/* utility */
+#include "shared.h"             /* MAX_LEN_ADDR */
+#include "support.h"            /* bool type */
 #include "timing.h"
 
+/* common */
 #include "fc_types.h"
 
-struct hash_table;
+struct genhash;
 struct timer_list;
+struct conn_pattern_list;
 
 #define MAX_LEN_PACKET   4096
 
@@ -43,25 +47,32 @@ struct timer_list;
 #define MAX_LEN_CAPSTR    512
 #define MAX_LEN_PASSWORD  512 /* do not change this under any circumstances */
 
-/**************************************************************************
+/****************************************************************************
   Command access levels for client-side use; at present, they are only
   used to control access to server commands typed at the client chatline.
-***************************************************************************/
-enum cmdlevel_id {    /* access levels for users to issue commands        */
-  ALLOW_NONE = 0,     /* user may issue no commands at all                */
-  ALLOW_INFO,         /* informational or observer commands only          */
-  ALLOW_BASIC,        /* user may issue basic player commands             */
-  ALLOW_CTRL,         /* user may issue commands that affect game & users */
-                      /* (starts a vote if the user's level is 'basic')   */
-  ALLOW_ADMIN,        /* user may issue commands that affect the server   */
-  ALLOW_HACK,         /* user may issue *all* commands - dangerous!       */
-
-  ALLOW_NUM,          /* the number of levels                             */
-  ALLOW_UNRECOGNIZED  /* used as a failure return code                    */
-};
-/*  the set command is a special case:                                    */
-/*    - ALLOW_CTRL is required for SSET_TO_CLIENT options                 */
-/*    - ALLOW_HACK is required for SSET_SERVER_ONLY options               */
+****************************************************************************/
+#define SPECENUM_NAME cmdlevel
+/* User may issue no commands at all. */
+#define SPECENUM_VALUE0 ALLOW_NONE
+#define SPECENUM_VALUE0NAME "none"
+/* Informational or observer commands only. */
+#define SPECENUM_VALUE1 ALLOW_INFO
+#define SPECENUM_VALUE1NAME "info"
+/* User may issue basic player commands. */
+#define SPECENUM_VALUE2 ALLOW_BASIC
+#define SPECENUM_VALUE2NAME "basic"
+/* User may issue commands that affect game & users
+ * (starts a vote if the user's level is 'basic'). */
+#define SPECENUM_VALUE3 ALLOW_CTRL
+#define SPECENUM_VALUE3NAME "ctrl"
+/* User may issue commands that affect the server. */
+#define SPECENUM_VALUE4 ALLOW_ADMIN
+#define SPECENUM_VALUE4NAME "admin"
+/* User may issue *all* commands - dangerous! */
+#define SPECENUM_VALUE5 ALLOW_HACK
+#define SPECENUM_VALUE5NAME "hack"
+#define SPECENUM_COUNT CMDLEVEL_COUNT
+#include "specenum_gen.h"
 
 /***************************************************************************
   On the distinction between nations(formerly races), players, and users,
@@ -113,6 +124,7 @@ struct connection {
   int sock;
   bool used;
   bool established;		/* have negotiated initial packets */
+  char *closing_reason;
 
   /* connection is "observer", not controller; may be observing
    * specific player, or all (implementation incomplete).
@@ -143,13 +155,7 @@ struct connection {
    * "access_level" stores the current access level of the client
    * corresponding to this connection.
    */
-  enum cmdlevel_id access_level;
-
-  /* 
-   * Something has occurred that means the connection should be
-   * closed, but the closing has been postponed. 
-   */
-  bool delayed_disconnect;
+  enum cmdlevel access_level;
 
   void (*notify_of_writable_data) (struct connection * pc,
                                    bool data_available_and_socket_full);
@@ -206,7 +212,14 @@ struct connection {
     char ipaddr[MAX_LEN_ADDR];
 
     /* The access level initially given to the client upon connection. */
-    enum cmdlevel_id granted_access_level;
+    enum cmdlevel granted_access_level;
+
+    /* The list of ignored connection patterns. */
+    struct conn_pattern_list *ignore_list;
+
+    /* Something has occurred that means the connection should be closed,
+     * but the closing has been postponed. */
+    bool is_closing;
   } server;
 
   /*
@@ -226,8 +239,8 @@ struct connection {
 				  int packet_type, int size,
 				  int request_id);
   struct {
-    struct hash_table **sent;
-    struct hash_table **received;
+    struct genhash **sent;
+    struct genhash **received;
     int *variant;
   } phs;
 
@@ -244,18 +257,14 @@ struct connection {
 };
 
 
-const char *cmdlevel_name(enum cmdlevel_id lvl);
-enum cmdlevel_id cmdlevel_named(const char *token);
-
-
-typedef void (*CLOSE_FUN) (struct connection *pc);
-void close_socket_set_callback(CLOSE_FUN fun);
-CLOSE_FUN close_socket_get_callback(void);
+typedef void (*conn_close_fn_t) (struct connection *pconn);
+void connections_set_close_callback(conn_close_fn_t func);
+void connection_close(struct connection *pconn, const char *reason);
 
 int read_socket_data(int sock, struct socket_packet_buffer *buffer);
 void flush_connection_send_buffer_all(struct connection *pc);
-void send_connection_data(struct connection *pc, const unsigned char *data,
-			  int len);
+bool connection_send_data(struct connection *pconn,
+                          const unsigned char *data, int len);
 
 void connection_do_buffer(struct connection *pc);
 void connection_do_unbuffer(struct connection *pc);
@@ -263,21 +272,27 @@ void connection_do_unbuffer(struct connection *pc);
 void conn_list_do_buffer(struct conn_list *dest);
 void conn_list_do_unbuffer(struct conn_list *dest);
 
-struct connection *find_conn_by_user(const char *user_name);
-struct connection *find_conn_by_user_prefix(const char *user_name,
-                                             enum m_pre_result *result);
-struct connection *find_conn_by_id(int id);
+struct connection *conn_by_user(const char *user_name);
+struct connection *conn_by_user_prefix(const char *user_name,
+                                       enum m_pre_result *result);
+struct connection *conn_by_number(int id);
 
 struct socket_packet_buffer *new_socket_packet_buffer(void);
 void connection_common_init(struct connection *pconn);
 void connection_common_close(struct connection *pconn);
 void free_compression_queue(struct connection *pconn);
-void conn_clear_packet_cache(struct connection *pconn);
+void conn_reset_delta_state(struct connection *pconn);
+
+void conn_compression_freeze(struct connection *pconn);
+bool conn_compression_thaw(struct connection *pconn);
+bool conn_compression_frozen(const struct connection *pconn);
+void conn_list_compression_freeze(const struct conn_list *pconn_list);
+void conn_list_compression_thaw(const struct conn_list *pconn_list);
 
 const char *conn_description(const struct connection *pconn);
 bool conn_controls_player(const struct connection *pconn);
 bool conn_is_global_observer(const struct connection *pconn);
-enum cmdlevel_id conn_get_access(const struct connection *pconn);
+enum cmdlevel conn_get_access(const struct connection *pconn);
 
 struct player;
 struct player *conn_get_player(const struct connection *pconn);
@@ -289,6 +304,40 @@ int get_next_request_id(int old_request_id);
 
 extern const char blank_addr_str[];
 
-extern int delayed_disconnect;
+
+/* Connection patterns. */
+struct conn_pattern;
+
+#define SPECLIST_TAG conn_pattern
+#define SPECLIST_TYPE struct conn_pattern
+#include "speclist.h"
+#define conn_pattern_list_iterate(plist, ppatern) \
+  TYPED_LIST_ITERATE(struct conn_pattern, plist, ppatern)
+#define conn_pattern_list_iterate_end LIST_ITERATE_END
+
+#define SPECENUM_NAME conn_pattern_type
+#define SPECENUM_VALUE0 CPT_USER
+#define SPECENUM_VALUE0NAME "user"
+#define SPECENUM_VALUE1 CPT_HOST
+#define SPECENUM_VALUE1NAME "host"
+#define SPECENUM_VALUE2 CPT_IP
+#define SPECENUM_VALUE2NAME "ip"
+#include "specenum_gen.h"
+
+struct conn_pattern *conn_pattern_new(enum conn_pattern_type type,
+                                      const char *wildcard);
+void conn_pattern_destroy(struct conn_pattern *ppattern);
+
+bool conn_pattern_match(const struct conn_pattern *ppattern,
+                        const struct connection *pconn);
+bool conn_pattern_list_match(const struct conn_pattern_list *plist,
+                             const struct connection *pconn);
+
+size_t conn_pattern_to_string(const struct conn_pattern *ppattern,
+                              char *buf, size_t buf_len);
+struct conn_pattern *conn_pattern_from_string(const char *pattern,
+                                              enum conn_pattern_type prefer,
+                                              char *error_buf,
+                                              size_t error_buf_len);
 
 #endif  /* FC__CONNECTION_H */
