@@ -156,7 +156,13 @@ void game_remove_unit(struct unit *punit)
 {
   struct city *pcity;
 
-  pcity = player_city_by_number(unit_owner(punit), punit->homecity);
+  /* It's possible that during city transfer homecity/unit owner
+   * information is inconsistent, and client then tries to remove
+   * now unseen unit so that homecity is not in the list of cities
+   * of the player (seemingly) owning the unit.
+   * Thus cannot use player_city_by_number() here, but have to
+   * consider cities of all players. */
+  pcity = game_city_by_number(punit->homecity);
   if (pcity) {
     unit_list_remove(pcity->units_supported, punit);
 
@@ -299,7 +305,7 @@ static void game_defaults(void)
   game.info.team_pooled_research = GAME_DEFAULT_TEAM_POOLED_RESEARCH;
   game.info.tech             = GAME_DEFAULT_TECHLEVEL;
   game.info.timeout          = GAME_DEFAULT_TIMEOUT;
-  game.info.trademindist     = GAME_DEFAULT_TRADEMINDIST;
+  game.info.trademindist_new = GAME_DEFAULT_TRADEMINDIST;
   game.info.trading_city     = GAME_DEFAULT_TRADING_CITY;
   game.info.trading_gold     = GAME_DEFAULT_TRADING_GOLD;
   game.info.trading_tech     = GAME_DEFAULT_TRADING_TECH;
