@@ -331,7 +331,7 @@ instead of just testing if the returning package is NULL as we sometimes
 return a NULL packet even if everything is OK (receive_packet_goto_route).
 **************************************************************************/
 void *get_packet_from_connection(struct connection *pc,
-				 enum packet_type *ptype, bool * presult)
+				 enum packet_type *ptype, bool *presult)
 {
   int len_read;
   int whole_packet_len;
@@ -342,8 +342,8 @@ void *get_packet_from_connection(struct connection *pc,
   struct data_in din;
 #ifdef USE_COMPRESSION
   bool compressed_packet = FALSE;
-  int header_size = 0;
 #endif
+  int header_size = 3;
 
   *presult = FALSE;
 
@@ -386,6 +386,14 @@ void *get_packet_from_connection(struct connection *pc,
     return NULL;		/* not all data has been read */
   }
 
+  if (whole_packet_len < header_size) {
+    log_verbose("The packet size is reported to be less than header alone. "
+                "The connection will be closed now.");
+    connection_close(pc, _("illegal packet size"));
+
+    return NULL;
+  }
+
 #ifdef USE_COMPRESSION
   if (compressed_packet) {
     uLong compressed_size = whole_packet_len - header_size;
