@@ -167,7 +167,8 @@ struct packet_game_info {
   int tech_upkeep_style;
   int timeout;
   bool tired_attack;
-  int trademindist;
+  int trademindist_old;
+  int trademindist_new;
   bool trading_city;
   bool trading_gold;
   bool trading_tech;
@@ -875,6 +876,10 @@ struct packet_ruleset_terrain_control {
   int fallout_tile_penalty[O_LAST];
 };
 
+struct packet_rulesets_ready {
+  char __dummy;			/* to avoid malloc(0); */
+};
+
 struct packet_ruleset_nation_groups {
   int ngroups;
   char groups[MAX_NUM_NATION_GROUPS][MAX_LEN_NAME];
@@ -1523,6 +1528,7 @@ enum packet_type {
   PACKET_EDIT_PLAYER_VISION,
   PACKET_EDIT_GAME,
   PACKET_EDIT_OBJECT_CREATED,
+  PACKET_RULESETS_READY = 221,
   PACKET_ENDGAME_PLAYER = 223,
   PACKET_ENDGAME_REPORT_NEW,
 
@@ -1961,6 +1967,10 @@ struct packet_ruleset_terrain_control *receive_packet_ruleset_terrain_control(st
 int send_packet_ruleset_terrain_control(struct connection *pc, const struct packet_ruleset_terrain_control *packet);
 void lsend_packet_ruleset_terrain_control(struct conn_list *dest, const struct packet_ruleset_terrain_control *packet);
 
+struct packet_rulesets_ready *receive_packet_rulesets_ready(struct connection *pc, enum packet_type type);
+int send_packet_rulesets_ready(struct connection *pc);
+void lsend_packet_rulesets_ready(struct conn_list *dest);
+
 struct packet_ruleset_nation_groups *receive_packet_ruleset_nation_groups(struct connection *pc, enum packet_type type);
 int send_packet_ruleset_nation_groups(struct connection *pc, const struct packet_ruleset_nation_groups *packet);
 void lsend_packet_ruleset_nation_groups(struct conn_list *dest, const struct packet_ruleset_nation_groups *packet);
