@@ -117,7 +117,16 @@ struct terrain *tile_terrain(const struct tile *ptile)
 ****************************************************************************/
 void tile_set_terrain(struct tile *ptile, struct terrain *pterrain)
 {
+  /* The terrain change is valid if one of the following is TRUE:
+   * - pterrain is NULL (= unknown terrain)
+   * - ptile is a virtual tile
+   * - pterrain does not has the flag TER_NO_CITIES
+   * - there is no city on ptile.
+   * This should be read as: The terrain change is INVALID if a terrain with
+   * the flag TER_NO_CITIES is given for a real tile with a city (i.e. all
+   * check evaluate to TRUE). */
   fc_assert_msg(NULL == pterrain
+                || tile_virtual_check(ptile)
                 || !terrain_has_flag(pterrain, TER_NO_CITIES)
                 || NULL == tile_city(ptile),
                 "At (%d, %d), the terrain \"%s\" (nb %d) doesn't "
@@ -440,12 +449,27 @@ int tile_activity_base_time(const struct tile *ptile,
 /****************************************************************************
   Clear all infrastructure (man-made specials) from the tile.
 ****************************************************************************/
-static void tile_clear_infrastructure(struct tile *ptile)
+static void tile_clear_unsupported_infrastructure(struct tile *ptile)
 {
   int i;
+  bool city_present = tile_city(ptile) != NULL;
+  struct terrain *pterr = tile_terrain(ptile);
+  bool ocean = is_ocean(pterr);
 
   for (i = 0; infrastructure_specials[i] != S_LAST; i++) {
-    tile_clear_special(ptile, infrastructure_specials[i]);
+    switch (infrastructure_specials[i]) {
+    case S_ROAD:
+    case S_RAILROAD:
+      if (!city_present && pterr->road_time == 0) {
+	tile_clear_special(ptile, infrastructure_specials[i]);
+      }
+      break;
+    default:
+      if (ocean) {
+	tile_clear_special(ptile, infrastructure_specials[i]);
+      }
+      break;
+    }
   }
 }
 
@@ -466,9 +490,9 @@ static void tile_clear_dirtiness(struct tile *ptile)
 void tile_change_terrain(struct tile *ptile, struct terrain *pterrain)
 {
   tile_set_terrain(ptile, pterrain);
-  if (is_ocean(pterrain)) {
-    tile_clear_infrastructure(ptile);
+  tile_clear_unsupported_infrastructure(ptile);
 
+  if (is_ocean(pterrain)) {
     /* The code can't handle these specials in ocean. */
     tile_clear_special(ptile, S_RIVER);
     tile_clear_special(ptile, S_HUT);
@@ -493,6 +517,11 @@ void tile_change_terrain(struct tile *ptile, struct terrain *pterrain)
   base_type_iterate(pbase) {
     if (tile_has_base(ptile, pbase)
         && !is_native_tile_to_base(pbase, ptile)) {
+      if (fc_funcs->destroy_base != NULL) {
+	fc_funcs->destroy_base(ptile, pbase);
+      } else {
+	tile_remove_base(ptile, pbase);
+      }
       tile_remove_base(ptile, pbase);
     }
   } base_type_iterate_end;
@@ -818,6 +847,11 @@ struct tile *tile_virtual_new(const struct tile *ptile)
   vtile->spec_sprite = NULL;
 
   if (ptile) {
+    /* Used by is_city_center to give virtual tiles the output bonuses
+     * they deserve */
+    vtile->x = ptile->x;
+    vtile->y = ptile->y;
+
     /* Copy all but the unit list. */
     tile_special_type_iterate(spe) {
       if (BV_ISSET(ptile->special, spe)) {
@@ -825,6 +859,10 @@ struct tile *tile_virtual_new(const struct tile *ptile)
       }
     } tile_special_type_iterate_end;
 
+    if (BV_ISSET(ptile->special, S_RESOURCE_VALID)) {
+      BV_SET(vtile->special, S_RESOURCE_VALID);
+    }
+
     base_type_iterate(pbase) {
       if (BV_ISSET(ptile->bases, base_number(pbase))) {
         BV_SET(vtile->bases, base_number(pbase));
@@ -877,3 +915,39 @@ void tile_virtual_destroy(struct tile *vtile)
 
   free(vtile);
 }
+
+/****************************************************************************
+  Check if the given tile is a virtual one or not.
+****************************************************************************/
+bool tile_virtual_check(struct tile *vtile)
+{
+  int tindex;
+
+  if (!vtile || map_is_empty()) {
+    return FALSE;
+  }
+
+  tindex = tile_index(vtile);
+
+  if (tindex == -1) {
+    /* This is a virtual tile. */
+    return TRUE;
+  }
+
+  fc_assert_ret_val(0 <= tindex && tindex < map_num_tiles(), FALSE);
+
+  return (vtile != map.tiles + tindex);
+}
+
+/****************************************************************************
+  Returns key that should be used when storing tile to hash or when
+  retrieving it from there.
+****************************************************************************/
+void *tile_hash_key(const struct tile *ptile)
+{
+  void *key = 0; /* Initialize whole sizeof(void *) */
+
+  key = FC_INT_TO_PTR(ptile->index);
+
+  return key;
+}
