@@ -15,7 +15,6 @@
 #include <config.h>
 #endif
 
-#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -54,42 +53,45 @@
 **************************************************************************/
 void handle_city_name_suggestion_req(struct player *pplayer, int unit_id)
 {
-  struct unit *punit = player_find_unit_by_id(pplayer, unit_id);
-  enum add_build_city_result res;
+  struct unit *punit = player_unit_by_number(pplayer, unit_id);
+  enum unit_add_build_city_result res;
 
   if (NULL == punit) {
     /* Probably died or bribed. */
-    freelog(LOG_VERBOSE, "handle_city_name_suggestion_req() invalid unit %d",
-            unit_id);
+    log_verbose("handle_city_name_suggestion_req() invalid unit %d",
+                unit_id);
     return;
   }
 
-  res = test_unit_add_or_build_city(punit);
+  res = unit_add_or_build_city_test(punit);
 
   switch (res) {
-  case AB_BUILD_OK:
-    freelog(LOG_VERBOSE, "handle_city_name_suggest_req(unit_pos (%d, %d))",
-            TILE_XY(unit_tile(punit)));
+  case UAB_BUILD_OK:
+    log_verbose("handle_city_name_suggest_req(unit_pos (%d, %d))",
+                TILE_XY(unit_tile(punit)));
     dlsend_packet_city_name_suggestion_info(pplayer->connections, unit_id,
         city_name_suggestion(pplayer, unit_tile(punit)));
     break;
 
-  case AB_NOT_BUILD_LOC:
-  case AB_NOT_BUILD_UNIT:
-  case AB_NO_MOVES_BUILD:
-    freelog(LOG_VERBOSE, "handle_city_name_suggest_req(unit_pos (%d, %d)): "
-            "cannot build there.", TILE_XY(unit_tile(punit)));
+  case UAB_BAD_CITY_TERRAIN:
+  case UAB_BAD_UNIT_TERRAIN:
+  case UAB_BAD_BORDERS:
+  case UAB_NO_MIN_DIST:
+  case UAB_NOT_BUILD_UNIT:
+  case UAB_NO_MOVES_BUILD:
+    log_verbose("handle_city_name_suggest_req(unit_pos (%d, %d)): "
+                "cannot build there.", TILE_XY(unit_tile(punit)));
     city_add_or_build_error(pplayer, punit, res);       /* Message. */
     break;
 
-  case AB_ADD_OK:
-  case AB_NOT_ADDABLE_UNIT:
-  case AB_NO_MOVES_ADD:
-  case AB_NOT_OWNER:
-  case AB_TOO_BIG:
-  case AB_NO_SPACE:
-    freelog(LOG_VERBOSE, "handle_city_name_suggest_req(unit_pos (%d, %d)): "
-            "there is already a city there.", TILE_XY(unit_tile(punit)));
+  case UAB_ADD_OK:
+  case UAB_NOT_ADDABLE_UNIT:
+  case UAB_NO_MOVES_ADD:
+  case UAB_NOT_OWNER:
+  case UAB_TOO_BIG:
+  case UAB_NO_SPACE:
+    log_verbose("handle_city_name_suggest_req(unit_pos (%d, %d)): "
+                "there is already a city there.", TILE_XY(unit_tile(punit)));
     /* Ignoring. */
     break;
   }
@@ -102,7 +104,7 @@ void handle_city_change_specialist(struct player *pplayer, int city_id,
 				   Specialist_type_id from,
 				   Specialist_type_id to)
 {
-  struct city *pcity = player_find_city_by_id(pplayer, city_id);
+  struct city *pcity = player_city_by_number(pplayer, city_id);
 
   if (!pcity) {
     return;
@@ -114,7 +116,7 @@ void handle_city_change_specialist(struct player *pplayer, int city_id,
       || pcity->specialists[from] == 0) {
     /* This could easily just be due to clicking faster on the specialist
      * than the server can cope with. */
-    freelog(LOG_VERBOSE, "Error in specialist change request from client.");
+    log_verbose("Error in specialist change request from client.");
     return;
   }
 
@@ -130,33 +132,32 @@ void handle_city_change_specialist(struct player *pplayer, int city_id,
 ...
 **************************************************************************/
 void handle_city_make_specialist(struct player *pplayer, int city_id,
-				 int worker_x, int worker_y)
+                                 int worker_x, int worker_y)
 {
   struct tile *ptile;
   struct tile *pcenter;
-  struct city *pcity = player_find_city_by_id(pplayer, city_id);
+  struct city *pcity = player_city_by_number(pplayer, city_id);
+  int city_radius_sq;
 
   if (NULL == pcity) {
     /* Probably lost. */
-    freelog(LOG_VERBOSE, "handle_city_make_specialist() bad city number %d.",
-            city_id);
+    log_verbose("handle_city_make_specialist() bad city number %d.",
+                city_id);
     return;
   }
-  
-  if (!is_valid_city_coords(worker_x, worker_y)) {
-    freelog(LOG_ERROR,
-            "handle_city_make_specialist() invalid city map {%d,%d} \"%s\".",
-            worker_x, worker_y,
-            city_name(pcity));
+
+  city_radius_sq = city_map_radius_sq_get(pcity);
+  if (!is_valid_city_coords(city_radius_sq, worker_x, worker_y)) {
+    log_error("handle_city_make_specialist() invalid city map {%d,%d} "
+              "\"%s\".", worker_x, worker_y, city_name(pcity));
     return;
   }
   pcenter = city_tile(pcity);
 
-  if (NULL == (ptile = city_map_to_tile(pcenter, worker_x, worker_y))) {
-    freelog(LOG_VERBOSE,
-            "handle_city_make_specialist() unavailable city map {%d,%d} \"%s\".",
-            worker_x, worker_y,
-            city_name(pcity));
+  if (NULL == (ptile = city_map_to_tile(pcenter, city_radius_sq, worker_x,
+                                        worker_y))) {
+    log_error("handle_city_make_specialist() unavailable city map {%d,%d} "
+              "\"%s\".", worker_x, worker_y, city_name(pcity));
     return;
   }
 
@@ -167,15 +168,13 @@ void handle_city_make_specialist(struct player *pplayer, int city_id,
   }
 
   if (tile_worked(ptile) == pcity) {
-    city_map_update_empty(pcity, ptile, worker_x, worker_y);
+    city_map_update_empty(pcity, ptile);
     pcity->specialists[DEFAULT_SPECIALIST]++;
     city_refresh(pcity);
     sync_cities();
   } else {
-    freelog(LOG_VERBOSE,
-            "handle_city_make_specialist() not working {%d,%d} \"%s\".",
-            worker_x, worker_y,
-            city_name(pcity));
+    log_verbose("handle_city_make_specialist() not working {%d,%d} \"%s\".",
+                worker_x, worker_y, city_name(pcity));
   }
 
   sanity_check_city(pcity);
@@ -189,29 +188,26 @@ void handle_city_make_worker(struct player *pplayer, int city_id,
 {
   struct tile *ptile;
   struct tile *pcenter;
-  struct city *pcity = player_find_city_by_id(pplayer, city_id);
+  struct city *pcity = player_city_by_number(pplayer, city_id);
+  int city_radius_sq = city_map_radius_sq_get(pcity);
 
   if (NULL == pcity) {
     /* Probably lost. */
-    freelog(LOG_VERBOSE, "handle_city_make_worker() bad city number %d.",
-            city_id);
+    log_verbose("handle_city_make_worker() bad city number %d.",city_id);
     return;
   }
 
-  if (!is_valid_city_coords(worker_x, worker_y)) {
-    freelog(LOG_ERROR,
-            "handle_city_make_worker() invalid city map {%d,%d} \"%s\".",
-            worker_x, worker_y,
-            city_name(pcity));
+  if (!is_valid_city_coords(city_radius_sq, worker_x, worker_y)) {
+    log_error("handle_city_make_worker() invalid city map {%d,%d} "
+              "\"%s\".", worker_x, worker_y, city_name(pcity));
     return;
   }
   pcenter = city_tile(pcity);
 
-  if (NULL == (ptile = city_map_to_tile(pcenter, worker_x, worker_y))) {
-    freelog(LOG_VERBOSE,
-            "handle_city_make_worker() unavailable city map {%d,%d} \"%s\".",
-            worker_x, worker_y,
-            city_name(pcity));
+  if (NULL == (ptile = city_map_to_tile(pcenter, city_radius_sq, worker_x,
+                                        worker_y))) {
+    log_error("handle_city_make_worker() unavailable city map {%d,%d} "
+              "\"%s\".", worker_x, worker_y, city_name(pcity));
     return;
   }
 
@@ -222,30 +218,24 @@ void handle_city_make_worker(struct player *pplayer, int city_id,
   }
 
   if (tile_worked(ptile) == pcity) {
-    freelog(LOG_VERBOSE,
-            "handle_city_make_worker() already working {%d,%d} \"%s\".",
-            worker_x, worker_y,
-            city_name(pcity));
+    log_verbose("handle_city_make_worker() already working {%d,%d} \"%s\".",
+                worker_x, worker_y, city_name(pcity));
     return;
   }
 
   if (0 == city_specialists(pcity)) {
-    freelog(LOG_VERBOSE,
-            "handle_city_make_worker() no specialists {%d,%d} \"%s\".",
-            worker_x, worker_y,
-            city_name(pcity));
+    log_verbose("handle_city_make_worker() no specialists {%d,%d} \"%s\".",
+                worker_x, worker_y, city_name(pcity));
     return;
   }
 
   if (!city_can_work_tile(pcity, ptile)) {
-    freelog(LOG_VERBOSE,
-            "handle_city_make_worker() cannot work here {%d,%d} \"%s\".",
-            worker_x, worker_y,
-            city_name(pcity));
+    log_verbose("handle_city_make_worker() cannot work here {%d,%d} \"%s\".",
+                worker_x, worker_y, city_name(pcity));
     return;
   }
 
-  city_map_update_worker(pcity, ptile, worker_x, worker_y);
+  city_map_update_worker(pcity, ptile);
 
   specialist_type_iterate(i) {
     if (pcity->specialists[i] > 0) {
@@ -265,6 +255,7 @@ void handle_city_make_worker(struct player *pplayer, int city_id,
 void really_handle_city_sell(struct player *pplayer, struct city *pcity,
 			     struct impr_type *pimprove)
 {
+  int price;
   if (pcity->did_sell) {
     notify_player(pplayer, pcity->tile, E_BAD_COMMAND, ftc_server,
 		  _("You have already sold something here this turn."));
@@ -275,18 +266,19 @@ void really_handle_city_sell(struct player *pplayer, struct city *pcity,
     return;
 
   pcity->did_sell=TRUE;
+  price = impr_sell_gold(pimprove);
   notify_player(pplayer, pcity->tile, E_IMP_SOLD, ftc_server,
-                _("You sell %s in %s for %d gold."), 
+                PL_("You sell %s in %s for %d gold.",
+                    "You sell %s in %s for %d gold.", price),
                 improvement_name_translation(pimprove),
-                city_link(pcity),
-                impr_sell_gold(pimprove));
+                city_link(pcity), price);
   do_sell_building(pplayer, pcity, pimprove);
 
   city_refresh(pcity);
 
   /* If we sold the walls the other players should see it */
   send_city_info(NULL, pcity);
-  send_player_info(pplayer, pplayer);
+  send_player_info_c(pplayer, pplayer->connections);
 }
 
 /**************************************************************************
@@ -294,7 +286,7 @@ void really_handle_city_sell(struct player *pplayer, struct city *pcity,
 **************************************************************************/
 void handle_city_sell(struct player *pplayer, int city_id, int build_id)
 {
-  struct city *pcity = player_find_city_by_id(pplayer, city_id);
+  struct city *pcity = player_city_by_number(pplayer, city_id);
   struct impr_type *pimprove = improvement_by_number(build_id);
 
   if (!pcity || !pimprove) {
@@ -312,7 +304,7 @@ void really_handle_city_buy(struct player *pplayer, struct city *pcity)
 
   /* This function corresponds to city_can_buy() in the client. */
 
-  assert(pcity && player_owns_city(pplayer, pcity));
+  fc_assert_ret(pcity && player_owns_city(pplayer, pcity));
  
   if (pcity->turn_founded == game.info.turn) {
     notify_player(pplayer, pcity->tile, E_BAD_COMMAND, ftc_server,
@@ -347,9 +339,19 @@ void really_handle_city_buy(struct player *pplayer, struct city *pcity)
   if (cost > pplayer->economic.gold) {
     /* In case something changed while player tried to buy, or player 
      * tried to cheat! */
+    /* Split into two to allow localization of two pluralisations. */
+    char buf[MAX_LEN_MSG];
+    /* TRANS: This whole string is only ever used when included in one
+     * other string (search for this string to find it). */
+    fc_snprintf(buf, ARRAY_SIZE(buf), PL_("%d gold required.",
+                                          "%d gold required.",
+                                          cost), cost);
     notify_player(pplayer, pcity->tile, E_BAD_COMMAND, ftc_server,
-                  _("%d gold required.  You only have %d gold."), cost,
-                  pplayer->economic.gold);
+                  /* TRANS: %s is a pre-pluralised string:
+                   * "%d gold required." */
+                  PL_("%s You only have %d gold.",
+                      "%s You only have %d gold.", pplayer->economic.gold),
+                  buf, pplayer->economic.gold);
     return;
   }
 
@@ -362,10 +364,24 @@ void really_handle_city_buy(struct player *pplayer, struct city *pcity)
     pcity->did_buy = TRUE;	/* !PS: no need to set buy flag otherwise */
   }
   city_refresh(pcity);
-  
+
+  if (VUT_UTYPE == pcity->production.kind) {
+    notify_player(pplayer, pcity->tile, E_UNIT_BUY, ftc_server,
+                  /* TRANS: bought an unit. */
+                  _("You bought %s in %s."),
+                  utype_rule_name(pcity->production.value.utype),
+                  city_name(pcity));
+  } else if (VUT_IMPROVEMENT == pcity->production.kind) {
+    notify_player(pplayer, pcity->tile, E_IMP_BUY, ftc_server,
+                  /* TRANS: bought an improvement .*/
+                  _("You bought %s in %s."),
+                  improvement_name_translation(
+                    pcity->production.value.building), city_name(pcity));
+  }
+
   conn_list_do_buffer(pplayer->connections);
   send_city_info(pplayer, pcity);
-  send_player_info(pplayer,pplayer);
+  send_player_info_c(pplayer, pplayer->connections);
   conn_list_do_unbuffer(pplayer->connections);
 }
 
@@ -375,7 +391,7 @@ void really_handle_city_buy(struct player *pplayer, struct city *pcity)
 void handle_city_worklist(struct player *pplayer, int city_id,
                           const struct worklist *worklist)
 {
-  struct city *pcity = player_find_city_by_id(pplayer, city_id);
+  struct city *pcity = player_city_by_number(pplayer, city_id);
 
   if (!pcity) {
     return;
@@ -391,7 +407,7 @@ void handle_city_worklist(struct player *pplayer, int city_id,
 **************************************************************************/
 void handle_city_buy(struct player *pplayer, int city_id)
 {
-  struct city *pcity = player_find_city_by_id(pplayer, city_id);
+  struct city *pcity = player_city_by_number(pplayer, city_id);
 
   if (!pcity) {
     return;
@@ -406,7 +422,7 @@ void handle_city_buy(struct player *pplayer, int city_id)
 void handle_city_refresh(struct player *pplayer, int city_id)
 {
   if (city_id != 0) {
-    struct city *pcity = player_find_city_by_id(pplayer, city_id);
+    struct city *pcity = player_city_by_number(pplayer, city_id);
 
     if (!pcity) {
       return;
@@ -426,21 +442,18 @@ void handle_city_change(struct player *pplayer, int city_id,
 			int production_kind, int production_value)
 {
   struct universal prod;
-  struct city *pcity = player_find_city_by_id(pplayer, city_id);
+  struct city *pcity = player_city_by_number(pplayer, city_id);
 
-  if (production_kind < VUT_NONE || production_kind >= VUT_LAST) {
-    freelog(LOG_ERROR, "handle_city_change()"
-            " bad production_kind %d.",
-            production_kind);
+  if (!universals_n_is_valid(production_kind)) {
+    log_error("[%s] bad production_kind %d.", __FUNCTION__,
+              production_kind);
     prod.kind = VUT_NONE;
     return;
   } else {
     prod = universal_by_number(production_kind, production_value);
-    if (prod.kind < VUT_NONE || prod.kind >= VUT_LAST) {
-      freelog(LOG_ERROR, "handle_city_change()"
-              " production_kind %d with bad production_value %d.",
-              production_kind,
-              production_value);
+    if (!universals_n_is_valid(prod.kind)) {
+      log_error("[%s] production_kind %d with bad production_value %d.",
+                __FUNCTION__, production_kind, production_value);
       prod.kind = VUT_NONE;
       return;
     }
@@ -471,12 +484,13 @@ void handle_city_change(struct player *pplayer, int city_id,
   send_city_info(pplayer, pcity);
 }
 
-/**************************************************************************
-...
-**************************************************************************/
-void handle_city_rename(struct player *pplayer, int city_id, char *name)
+/****************************************************************************
+  'struct packet_city_rename' handler.
+****************************************************************************/
+void handle_city_rename(struct player *pplayer, int city_id,
+                        const char *name)
 {
-  struct city *pcity = player_find_city_by_id(pplayer, city_id);
+  struct city *pcity = player_city_by_number(pplayer, city_id);
   char message[1024];
 
   if (!pcity) {
@@ -501,7 +515,7 @@ void handle_city_rename(struct player *pplayer, int city_id, char *name)
 void handle_city_options_req(struct player *pplayer, int city_id,
 			     bv_city_options options)
 {
-  struct city *pcity = player_find_city_by_id(pplayer, city_id);
+  struct city *pcity = player_city_by_number(pplayer, city_id);
 
   if (!pcity) {
     return;
