@@ -577,7 +577,9 @@ static void transfer_unit(struct unit *punit, struct city *tocity,
 		    city_link(tocity));
     }
   } else {
-    struct city *in_city = tile_city(punit->tile);
+    struct tile *utile = unit_tile(punit);
+    struct city *in_city = tile_city(utile);
+
     if (in_city) {
       log_verbose("Transferred %s in %s from %s to %s",
                   unit_rule_name(punit), city_name(in_city),
@@ -620,10 +622,11 @@ static void transfer_unit(struct unit *punit, struct city *tocity,
                       nation_adjective_for_player(to_player),
                       city_link(tocity));
       }
-      from_player->score.units_lost++;
-      wipe_unit(punit);
+      wipe_unit(punit, TRUE, NULL);
       return;
     }
+
+    maybe_make_contact(utile, to_player);
   }
   unit_change_homecity_handling(punit, tocity);
 }
@@ -710,8 +713,7 @@ void transfer_city_units(struct player *pplayer, struct player *pvictim,
                       _("%s lost along with control of %s."),
                       unit_tile_link(vunit), name);
       }
-      unit_owner(vunit)->score.units_lost++;
-      wipe_unit(vunit);
+      wipe_unit(vunit, TRUE, NULL);
     }
   } unit_list_iterate_safe_end;
 
@@ -1426,8 +1428,7 @@ void remove_city(struct city *pcity)
                       "get out, and it was therefore lost."),
                     city_link(pcity),
                     unit_tile_link(punit));
-      unit_owner(punit)->score.units_lost++;
-      wipe_unit(punit);
+      wipe_unit(punit, TRUE, NULL);
     }
   } unit_list_iterate_safe_end;
 
@@ -1437,8 +1438,7 @@ void remove_city(struct city *pcity)
 
   /* Any remaining supported units are destroyed */
   unit_list_iterate_safe(pcity->units_supported, punit) {
-    unit_owner(punit)->score.units_lost++;
-    wipe_unit(punit);
+    wipe_unit(punit, TRUE, NULL);
   } unit_list_iterate_safe_end;
 
   if (!city_exist(id)) {
