@@ -220,7 +220,7 @@ static struct command commands[] = {
   },
   {"metamessage", ALLOW_CTRL,
    /* TRANS: translate text between <> only */
-   N_("metainfo <meta-line>"),
+   N_("metamessage <meta-line>"),
    N_("Set metaserver info line."),
    N_("Set user defined metaserver info line. If parameter is omitted, "
       "previously set metamessage will be removed. For most of the time "
@@ -230,7 +230,7 @@ static struct command commands[] = {
   },
   {"metapatches", ALLOW_HACK,
    /* TRANS: translate text between <> only */
-   N_("metapatch <meta-line>"),
+   N_("metapatches <meta-line>"),
    N_("Set metaserver patches line."), NULL,
    CMD_ECHO_ADMINS, VCF_NONE, 0
   },
@@ -529,7 +529,7 @@ static struct command commands[] = {
   {"lua", ALLOW_ADMIN,
    /* TRANS: translate text between <> only */
    N_("lua <script>"),
-   N_("Evaluate a line of freeciv script in the current game."), NULL,
+   N_("Evaluate a line of Freeciv script in the current game."), NULL,
    CMD_ECHO_ADMINS, VCF_NONE, 0
   },
   {"kick", ALLOW_CTRL,
