@@ -14,7 +14,7 @@
 #ifndef FC__COMMANDS_H
 #define FC__COMMANDS_H
 
-#include "connection.h"		/* for enum cmdlevel_id */
+#include "connection.h"         /* enum cmdlevel */
 
 enum cmd_echo {
   CMD_ECHO_NONE = 0,
@@ -71,6 +71,8 @@ enum command_id {
   CMD_FIRSTLEVEL,
   CMD_TIMEOUT,
   CMD_CANCELVOTE,
+  CMD_IGNORE,
+  CMD_UNIGNORE,
 
   /* potentially harmful: */
   CMD_END_GAME,
@@ -81,6 +83,8 @@ enum command_id {
   CMD_READ_SCRIPT,
   CMD_WRITE_SCRIPT,
   CMD_RESET,
+  CMD_LUA,
+  CMD_KICK,
 
   /* undocumented */
   CMD_RFCSTYLE,
@@ -100,7 +104,7 @@ const char *command_synopsis(const struct command *pcommand);
 const char *command_short_help(const struct command *pcommand);
 const char *command_extra_help(const struct command *pcommand);
 
-enum cmdlevel_id command_level(const struct command *pcommand);
+enum cmdlevel command_level(const struct command *pcommand);
 enum cmd_echo command_echo(const struct command *pcommand);
 int command_vote_flags(const struct command *pcommand);
 int command_vote_percent(const struct command *pcommand);
