@@ -18,6 +18,7 @@
 #include <stdio.h>
 
 /* utility */
+#include "bitvector.h"
 #include "fcintl.h"
 #include "log.h"
 #include "rand.h"
@@ -29,6 +30,7 @@
 #include "government.h"
 #include "movement.h"
 #include "player.h"
+#include "research.h"
 #include "unitlist.h"
 
 /* server */
@@ -37,7 +39,6 @@
 #include "cityturn.h"
 #include "diplhand.h"
 #include "diplomats.h"
-#include "gotohand.h"
 #include "notify.h"
 #include "plrhand.h"
 #include "techtools.h"
@@ -58,8 +59,11 @@ static bool diplomat_infiltrate_tile(struct player *pplayer, struct player *cpla
 				     struct unit *pdiplomat, struct tile *ptile);
 static void diplomat_escape(struct player *pplayer, struct unit *pdiplomat,
 			    const struct city *pcity);
-static void maybe_cause_incident(enum diplomat_actions action, struct player *offender,
-				 struct unit *victim_unit, struct city *victim_city);
+static void maybe_cause_incident(enum diplomat_actions action,
+                                 struct player *offender,
+                                 struct player *victim_player,
+                                 struct tile *victim_tile,
+                                 const char *victim_link);
 
 /******************************************************************************
   Poison a city's water supply.
@@ -88,7 +92,7 @@ void spy_poison(struct player *pplayer, struct unit *pdiplomat,
   if (!cplayer || !pplayers_at_war(pplayer, cplayer))
     return;
 
-  freelog (LOG_DEBUG, "poison: unit: %d", pdiplomat->id);
+  log_debug("poison: unit: %d", pdiplomat->id);
 
   /* If not a Spy, can't poison. */
   if (!unit_has_type_flag(pdiplomat, F_SPY))
@@ -100,7 +104,7 @@ void spy_poison(struct player *pplayer, struct unit *pdiplomat,
     return;
   }
 
-  freelog (LOG_DEBUG, "poison: infiltrated");
+  log_debug("poison: infiltrated");
 
   /* If city is too small, can't poison. */
   if (pcity->size < 2) {
@@ -110,11 +114,11 @@ void spy_poison(struct player *pplayer, struct unit *pdiplomat,
                     " supply in %s."),
                   unit_link(pdiplomat),
                   city_link(pcity));
-    freelog (LOG_DEBUG, "poison: target city too small");
+    log_debug("poison: target city too small");
     return;
   }
 
-  freelog (LOG_DEBUG, "poison: succeeded");
+  log_debug("poison: succeeded");
 
   /* Poison people! */
   city_reduce_size(pcity, 1, pplayer);
@@ -135,7 +139,8 @@ void spy_poison(struct player *pplayer, struct unit *pdiplomat,
   send_city_info(NULL, pcity);
 
   /* this may cause a diplomatic incident */
-  maybe_cause_incident(SPY_POISON, pplayer, NULL, pcity);
+  maybe_cause_incident(SPY_POISON, pplayer, cplayer,
+                       city_tile(pcity), city_link(pcity));
 
   /* Now lets see if the spy survives. */
   diplomat_escape(pplayer, pdiplomat, pcity);
@@ -167,7 +172,7 @@ void diplomat_investigate(struct player *pplayer, struct unit *pdiplomat,
   if ((cplayer == pplayer) || !cplayer)
     return;
 
-  freelog (LOG_DEBUG, "investigate: unit: %d", pdiplomat->id);
+  log_debug("investigate: unit: %d", pdiplomat->id);
 
   /* Do It... */
   update_dumb_city(pplayer, pcity);
@@ -192,7 +197,9 @@ void diplomat_investigate(struct player *pplayer, struct unit *pdiplomat,
   /* Send city info to investigator's player.
      As this is a special case we bypass send_city_info. */
   package_city(pcity, &city_packet, TRUE);
-  lsend_packet_city_info(pplayer->connections, &city_packet);
+  /* We need to force to send the packet to ensure the client will receive
+   * something and popup the city dialog. */
+  lsend_packet_city_info(pplayer->connections, &city_packet, TRUE);
 
   /* Charge a nominal amount of movement for this. */
   (pdiplomat->moves_left)--;
@@ -201,11 +208,12 @@ void diplomat_investigate(struct player *pplayer, struct unit *pdiplomat,
   }
 
   /* this may cause a diplomatic incident */
-  maybe_cause_incident(DIPLOMAT_INVESTIGATE, pplayer, NULL, pcity);
+  maybe_cause_incident(DIPLOMAT_INVESTIGATE, pplayer, cplayer,
+                       city_tile(pcity), city_link(pcity));
 
   /* Spies always survive. Diplomats never do. */
   if (!unit_has_type_flag(pdiplomat, F_SPY)) {
-    wipe_unit(pdiplomat);
+    wipe_unit(pdiplomat, FALSE, NULL);
   } else {
     send_unit_info (pplayer, pdiplomat);
   }
@@ -265,7 +273,7 @@ void diplomat_embassy(struct player *pplayer, struct unit *pdiplomat,
   if ((cplayer == pplayer) || !cplayer)
     return;
 
-  freelog (LOG_DEBUG, "embassy: unit: %d", pdiplomat->id);
+  log_debug("embassy: unit: %d", pdiplomat->id);
 
   /* Check for Barbarian response. */
   if (get_player_bonus(cplayer, EFT_NO_DIPLOMACY)) {
@@ -275,11 +283,11 @@ void diplomat_embassy(struct player *pplayer, struct unit *pdiplomat,
                   unit_tile_link(pdiplomat),
                   city_link(pcity),
                   nation_plural_for_player(cplayer));
-    wipe_unit(pdiplomat);
+    wipe_unit(pdiplomat, TRUE, cplayer);
     return;
   }
 
-  freelog (LOG_DEBUG, "embassy: succeeded");
+  log_debug("embassy: succeeded");
 
   establish_embassy(pplayer, cplayer);
 
@@ -301,11 +309,12 @@ void diplomat_embassy(struct player *pplayer, struct unit *pdiplomat,
   }
 
   /* this may cause a diplomatic incident */
-  maybe_cause_incident(DIPLOMAT_EMBASSY, pplayer, NULL, pcity);
+  maybe_cause_incident(DIPLOMAT_EMBASSY, pplayer, cplayer,
+                       city_tile(pcity), city_link(pcity));
 
   /* Spies always survive. Diplomats never do. */
   if (!unit_has_type_flag(pdiplomat, F_SPY)) {
-    wipe_unit(pdiplomat);
+    wipe_unit(pdiplomat, FALSE, NULL);
   } else {
     send_unit_info (pplayer, pdiplomat);
   }
@@ -338,7 +347,7 @@ void spy_sabotage_unit(struct player *pplayer, struct unit *pdiplomat,
   if (!uplayer || pplayers_allied(pplayer, uplayer))
     return;
 
-  freelog(LOG_DEBUG, "sabotage-unit: unit: %d", pdiplomat->id);
+  log_debug("sabotage-unit: unit: %d", pdiplomat->id);
 
   /* If not a Spy, can't sabotage unit. */
   if (!unit_has_type_flag(pdiplomat, F_SPY))
@@ -355,7 +364,7 @@ void spy_sabotage_unit(struct player *pplayer, struct unit *pdiplomat,
                   unit_link(pdiplomat),
                   nation_adjective_for_player(uplayer),
                   victim_link);
-    freelog(LOG_DEBUG, "sabotage-unit: unit has too few hit points");
+    log_debug("sabotage-unit: unit has too few hit points");
     return;
   }
 
@@ -365,7 +374,7 @@ void spy_sabotage_unit(struct player *pplayer, struct unit *pdiplomat,
     return;
   }
 
-  freelog(LOG_DEBUG, "sabotage-unit: succeeded");
+  log_debug("sabotage-unit: succeeded");
 
   /* Sabotage the unit by removing half its remaining hit points. */
   pvictim->hp /= 2;
@@ -386,7 +395,8 @@ void spy_sabotage_unit(struct player *pplayer, struct unit *pdiplomat,
                 nation_plural_for_player(pplayer));
 
   /* this may cause a diplomatic incident */
-  maybe_cause_incident(SPY_SABOTAGE_UNIT, pplayer, pvictim, NULL);
+  maybe_cause_incident(SPY_SABOTAGE_UNIT, pplayer, uplayer,
+                       unit_tile(pvictim), victim_link);
 
   /* Now lets see if the spy survives. */
   diplomat_escape(pplayer, pdiplomat, NULL);
@@ -415,17 +425,18 @@ void diplomat_bribe(struct player *pplayer, struct unit *pdiplomat,
   struct tile *victim_tile;
   int bribe_cost;
   int diplomat_id = pdiplomat->id;
-  struct unit *gained_unit = NULL;
 
   /* Fetch target unit's player.  Sanity checks. */
-  if (!pvictim)
+  if (!pvictim) {
     return;
+  }
   uplayer = unit_owner(pvictim);
   /* We might make it allowable in peace with a loss of reputation */
-  if (!uplayer || pplayers_allied(pplayer, uplayer))
+  if (!uplayer || pplayers_allied(pplayer, uplayer)) {
     return;
+  }
 
-  freelog(LOG_DEBUG, "bribe-unit: unit: %d", pdiplomat->id);
+  log_debug("bribe-unit: unit: %d", pdiplomat->id);
 
   /* Check for unit from a bribable government. */
   if (get_player_bonus(uplayer, EFT_UNBRIBABLE_UNITS)) {
@@ -435,9 +446,6 @@ void diplomat_bribe(struct player *pplayer, struct unit *pdiplomat,
     return;
   }
 
-  /* N.B.: unit_link always returns the same pointer. */
-  sz_strlcpy(victim_link, unit_link(pvictim));
-
   /* Get bribe cost, ignoring any previously saved value. */
   bribe_cost = unit_bribe_cost(pvictim);
 
@@ -447,8 +455,8 @@ void diplomat_bribe(struct player *pplayer, struct unit *pdiplomat,
                   E_MY_DIPLOMAT_FAILED, ftc_server,
                   _("You don't have enough gold to bribe the %s %s."),
                   nation_adjective_for_player(uplayer),
-                  victim_link);
-    freelog(LOG_DEBUG, "bribe-unit: not enough gold");
+                  unit_link(pvictim));
+    log_debug("bribe-unit: not enough gold");
     return;
   }
 
@@ -456,62 +464,39 @@ void diplomat_bribe(struct player *pplayer, struct unit *pdiplomat,
     notify_player(pplayer, unit_tile(pdiplomat),
                   E_MY_DIPLOMAT_FAILED, ftc_server,
                   _("You cannot bribe the %s!"),
-                  victim_link);
+                  unit_link(pvictim));
     return;
   }
 
-  freelog (LOG_DEBUG, "bribe-unit: succeeded");
+  log_debug("bribe-unit: succeeded");
 
-  /* Convert the unit to your cause. Fog is lifted in the create algorithm. */
-  gained_unit = create_unit_full(pplayer, pvictim->tile,
-                                 unit_type(pvictim), pvictim->veteran,
-                                 pdiplomat->homecity, pvictim->moves_left,
-                                 pvictim->hp, NULL);
+  victim_tile = pvictim->tile;
+  pvictim = unit_change_owner(pvictim, pplayer, pdiplomat->homecity);
 
-  /* Copy some more unit fields */
-  gained_unit->fuel        = pvictim->fuel;
-  gained_unit->paradropped = pvictim->paradropped;
-  gained_unit->birth_turn  = pvictim->birth_turn;
-
-  /* Inform owner about less than full fuel */
-  send_unit_info(pplayer, gained_unit);
+  /* N.B.: unit_link always returns the same pointer. As unit_change_owner()
+   * currently remove the old unit and replace by a new one (with a new id),
+   * we want to make link to the new unit. */
+  sz_strlcpy(victim_link, unit_link(pvictim));
 
   /* Notify everybody involved. */
-  notify_player(pplayer, unit_tile(pvictim), 
-                E_MY_DIPLOMAT_BRIBE, ftc_server,
+  notify_player(pplayer, victim_tile, E_MY_DIPLOMAT_BRIBE, ftc_server,
                 /* TRANS: <diplomat> ... <unit> */
                 _("Your %s succeeded in bribing the %s."),
-                unit_link(pdiplomat),
-                victim_link);
+                unit_link(pdiplomat), victim_link);
   if (maybe_make_veteran(pdiplomat)) {
     notify_unit_experience(pdiplomat);
   }
-  notify_player(uplayer, unit_tile(pvictim),
-                E_ENEMY_DIPLOMAT_BRIBE, ftc_server,
+  notify_player(uplayer, victim_tile, E_ENEMY_DIPLOMAT_BRIBE, ftc_server,
                 /* TRANS: <unit> ... <Poles> */
                 _("Your %s was bribed by the %s."),
-                victim_link,
-                nation_plural_for_player(pplayer));
+                victim_link, nation_plural_for_player(pplayer));
 
   /* This costs! */
   pplayer->economic.gold -= bribe_cost;
 
   /* This may cause a diplomatic incident */
-  maybe_cause_incident(DIPLOMAT_BRIBE, pplayer, pvictim, NULL);
-
-  /* update unit upkeep in the homecity of the victim */
-  if (pvictim->homecity > 0) {
-    /* update unit upkeep */
-    city_units_upkeep(game_find_city_by_number(pvictim->homecity));
-  }
-  /* update unit upkeep in the new homecity */
-  if (pvictim->homecity > 0) {
-    city_units_upkeep(game_find_city_by_number(gained_unit->homecity));
-  }
-
-  /* Be sure to wipe the converted unit! */
-  victim_tile = pvictim->tile;
-  wipe_unit(pvictim);
+  maybe_cause_incident(DIPLOMAT_BRIBE, pplayer, uplayer,
+                       victim_tile, victim_link);
 
   if (!unit_alive(diplomat_id)) {
     return;
@@ -521,12 +506,12 @@ void diplomat_bribe(struct player *pplayer, struct unit *pdiplomat,
   if (!unit_move_handling(pdiplomat, victim_tile, FALSE, FALSE)) {
     pdiplomat->moves_left = 0;
   }
-  if (player_find_unit_by_id(pplayer, diplomat_id)) {
-    send_unit_info (pplayer, pdiplomat);
+  if (NULL != player_unit_by_number(pplayer, diplomat_id)) {
+    send_unit_info(pplayer, pdiplomat);
   }
 
   /* Update clients. */
-  send_player_info(pplayer, NULL);
+  send_player_all_c(pplayer, NULL);
 }
 
 /****************************************************************************
@@ -578,8 +563,8 @@ void diplomat_get_tech(struct player *pplayer, struct unit *pdiplomat,
   
   if (technology == A_FUTURE) {
     if (player_invention_state(pplayer, A_FUTURE) != TECH_PREREQS_KNOWN
-        || get_player_research(pplayer)->future_tech >= 
-	   get_player_research(pplayer)->future_tech) {
+        || player_research_get(pplayer)->future_tech >= 
+	   player_research_get(pplayer)->future_tech) {
       return;
     }
   } else if (technology != A_UNSET) {
@@ -591,7 +576,7 @@ void diplomat_get_tech(struct player *pplayer, struct unit *pdiplomat,
     }
   }
 
-  freelog (LOG_DEBUG, "steal-tech: unit: %d", pdiplomat->id);
+  log_debug("steal-tech: unit: %d", pdiplomat->id);
 
   /* If not a Spy, do something random. */
   if (!unit_has_type_flag(pdiplomat, F_SPY)) {
@@ -604,34 +589,34 @@ void diplomat_get_tech(struct player *pplayer, struct unit *pdiplomat,
     return;
   }
 
-  freelog (LOG_DEBUG, "steal-tech: infiltrated");
+  log_debug("steal-tech: infiltrated");
 
   /* Check if the Diplomat/Spy succeeds with his/her task. */
   /* (Twice as difficult if target is specified.) */
   /* (If already stolen from, impossible for Diplomats and harder for Spies.) */
-  if (pcity->steal > 0 && !unit_has_type_flag(pdiplomat, F_SPY)) {
+  if (pcity->server.steal > 0 && !unit_has_type_flag(pdiplomat, F_SPY)) {
     /* Already stolen from: Diplomat always fails! */
     count = 1;
-    freelog (LOG_DEBUG, "steal-tech: difficulty: impossible");
+    log_debug("steal-tech: difficulty: impossible");
   } else {
     /* Determine difficulty. */
     count = 1;
     if (technology != A_UNSET) {
       count++;
     }
-    count += pcity->steal;
-    freelog (LOG_DEBUG, "steal-tech: difficulty: %d", count);
+    count += pcity->server.steal;
+    log_debug("steal-tech: difficulty: %d", count);
     /* Determine success or failure. */
     while (count > 0) {
-      if (myrand (100) >= game.info.diplchance) {
-	break;
+      if (fc_rand (100) >= game.server.diplchance) {
+        break;
       }
       count--;
     }
   }
   
   if (count > 0) {
-    if (pcity->steal > 0 && !unit_has_type_flag(pdiplomat, F_SPY)) {
+    if (pcity->server.steal > 0 && !unit_has_type_flag(pdiplomat, F_SPY)) {
       notify_player(pplayer, city_tile(pcity),
                     E_MY_DIPLOMAT_FAILED, ftc_server,
                     _("%s was expecting your attempt to steal technology "
@@ -653,8 +638,9 @@ void diplomat_get_tech(struct player *pplayer, struct unit *pdiplomat,
                   unit_tile_link(pdiplomat),
                   city_link(pcity));
     /* this may cause a diplomatic incident */
-    maybe_cause_incident(DIPLOMAT_STEAL, pplayer, NULL, pcity);
-    wipe_unit(pdiplomat);
+    maybe_cause_incident(DIPLOMAT_STEAL, pplayer, cplayer,
+                         city_tile(pcity), city_link(pcity));
+    wipe_unit(pdiplomat, TRUE, cplayer);
     return;
   } 
 
@@ -671,13 +657,14 @@ void diplomat_get_tech(struct player *pplayer, struct unit *pdiplomat,
   }
 
   /* Update stealing player's science progress and research fields */
-  send_player_info(pplayer, NULL);
- 
+  send_player_all_c(pplayer, NULL);
+
   /* Record the theft. */
-  (pcity->steal)++;
+  (pcity->server.steal)++;
 
   /* this may cause a diplomatic incident */
-  maybe_cause_incident(DIPLOMAT_STEAL, pplayer, NULL, pcity);
+  maybe_cause_incident(DIPLOMAT_STEAL, pplayer, cplayer,
+                       city_tile(pcity), city_link(pcity));
 
   /* Check if a spy survives her mission. Diplomats never do. */
   diplomat_escape(pplayer, pdiplomat, pcity);
@@ -714,14 +701,14 @@ void diplomat_incite(struct player *pplayer, struct unit *pdiplomat,
   if (!cplayer || pplayers_allied(cplayer, pplayer))
     return;
 
-  freelog (LOG_DEBUG, "incite: unit: %d", pdiplomat->id);
+  log_debug("incite: unit: %d", pdiplomat->id);
 
   /* See if the city is subvertable. */
   if (get_city_bonus(pcity, EFT_NO_INCITE) > 0) {
     notify_player(pplayer, city_tile(pcity),
                   E_MY_DIPLOMAT_FAILED, ftc_server,
                   _("You can't subvert this city."));
-    freelog (LOG_DEBUG, "incite: city is protected");
+    log_debug("incite: city is protected");
     return;
   }
 
@@ -734,7 +721,7 @@ void diplomat_incite(struct player *pplayer, struct unit *pdiplomat,
                   E_MY_DIPLOMAT_FAILED, ftc_server,
                   _("You don't have enough gold to subvert %s."),
                   city_link(pcity));
-    freelog(LOG_DEBUG, "incite: not enough gold");
+    log_debug("incite: not enough gold");
     return;
   }
 
@@ -744,10 +731,10 @@ void diplomat_incite(struct player *pplayer, struct unit *pdiplomat,
     return;
   }
 
-  freelog (LOG_DEBUG, "incite: infiltrated");
+  log_debug("incite: infiltrated");
 
   /* Check if the Diplomat/Spy succeeds with his/her task. */
-  if (myrand (100) >= game.info.diplchance) {
+  if (fc_rand (100) >= game.server.diplchance) {
     notify_player(pplayer, city_tile(pcity),
                   E_MY_DIPLOMAT_FAILED, ftc_server,
                   _("Your %s was caught in the attempt"
@@ -760,11 +747,11 @@ void diplomat_incite(struct player *pplayer, struct unit *pdiplomat,
                   nation_adjective_for_player(pplayer),
                   unit_tile_link(pdiplomat),
                   city_link(pcity));
-    wipe_unit(pdiplomat);
+    wipe_unit(pdiplomat, TRUE, cplayer);
     return;
   }
 
-  freelog (LOG_DEBUG, "incite: succeeded");
+  log_debug("incite: succeeded");
 
   /* Subvert the city to your cause... */
 
@@ -794,7 +781,8 @@ void diplomat_incite(struct player *pplayer, struct unit *pdiplomat,
   steal_a_tech (pplayer, cplayer, A_UNSET);
 
   /* this may cause a diplomatic incident */
-  maybe_cause_incident(DIPLOMAT_INCITE, pplayer, NULL, pcity);
+  maybe_cause_incident(DIPLOMAT_INCITE, pplayer, cplayer,
+                       city_tile(pcity), city_link(pcity));
 
   /* Transfer city and units supported by this city (that
      are within one square of the city) to the new owner. */
@@ -807,8 +795,8 @@ void diplomat_incite(struct player *pplayer, struct unit *pdiplomat,
   diplomat_escape(pplayer, pdiplomat, pcity);
 
   /* Update the players gold in the client */
-  send_player_info(pplayer, pplayer);
-}  
+  send_player_info_c(pplayer, pplayer->connections);
+}
 
 /**************************************************************************
   Sabotage enemy city's improvement or production.
@@ -843,15 +831,15 @@ void diplomat_sabotage(struct player *pplayer, struct unit *pdiplomat,
   if (!cplayer || !pplayers_at_war(pplayer, cplayer))
     return;
 
-  freelog (LOG_DEBUG, "sabotage: unit: %d", pdiplomat->id);
+  log_debug("sabotage: unit: %d", pdiplomat->id);
 
   /* If not a Spy, do something random. */
   if (!unit_has_type_flag(pdiplomat, F_SPY))
     improvement = B_LAST;
 
   /* Twice as difficult if target is specified. */
-  success_prob = (improvement >= B_LAST ? game.info.diplchance 
-                  : game.info.diplchance / 2); 
+  success_prob = (improvement >= B_LAST ? game.server.diplchance 
+                  : game.server.diplchance / 2); 
 
   /* Check if the Diplomat/Spy succeeds against defending Diplomats/Spies. */
   if (!diplomat_infiltrate_tile(pplayer, cplayer, pdiplomat, 
@@ -859,10 +847,10 @@ void diplomat_sabotage(struct player *pplayer, struct unit *pdiplomat,
     return;
   }
 
-  freelog (LOG_DEBUG, "sabotage: infiltrated");
+  log_debug("sabotage: infiltrated");
 
   /* Check if the Diplomat/Spy succeeds with his/her task. */
-  if (myrand (100) >= success_prob) {
+  if (fc_rand (100) >= success_prob) {
     notify_player(pplayer, city_tile(pcity),
                   E_MY_DIPLOMAT_FAILED, ftc_server,
                   _("Your %s was caught in the attempt"
@@ -874,11 +862,11 @@ void diplomat_sabotage(struct player *pplayer, struct unit *pdiplomat,
                   nation_adjective_for_player(pplayer),
                   unit_tile_link(pdiplomat),
                   city_link(pcity));
-    wipe_unit(pdiplomat);
+    wipe_unit(pdiplomat, TRUE, cplayer);
     return;
   }
 
-  freelog (LOG_DEBUG, "sabotage: succeeded");
+  log_debug("sabotage: succeeded");
 
   /* Examine the city for improvements to sabotage. */
   count = 0;
@@ -888,13 +876,13 @@ void diplomat_sabotage(struct player *pplayer, struct unit *pdiplomat,
     }
   } city_built_iterate_end;
 
-  freelog (LOG_DEBUG, "sabotage: count of improvements: %d", count);
+  log_debug("sabotage: count of improvements: %d", count);
 
   /* Determine the target (-1 is production). */
   if (improvement < 0) {
     /* If told to sabotage production, do so. */
     ptarget = NULL;
-    freelog (LOG_DEBUG, "sabotage: specified target production");
+    log_debug("sabotage: specified target production");
   } else if (improvement >= B_LAST) {
     /*
      * Pick random:
@@ -910,15 +898,15 @@ void diplomat_sabotage(struct player *pplayer, struct unit *pdiplomat,
                     city_link(pcity));
       diplomat_charge_movement(pdiplomat, pcity->tile);
       send_unit_info(pplayer, pdiplomat);
-      freelog(LOG_DEBUG, "sabotage: random: nothing to do");
+      log_debug("sabotage: random: nothing to do");
       return;
     }
-    if (count == 0 || myrand (2) == 1) {
+    if (count == 0 || fc_rand (2) == 1) {
       ptarget = NULL;
-      freelog (LOG_DEBUG, "sabotage: random: targeted production");
+      log_debug("sabotage: random: targeted production");
     } else {
       ptarget = NULL;
-      which = myrand (count);
+      which = fc_rand (count);
 
       city_built_iterate(pcity, pimprove) {
 	if (pimprove->sabotage > 0) {
@@ -932,11 +920,11 @@ void diplomat_sabotage(struct player *pplayer, struct unit *pdiplomat,
       } city_built_iterate_end;
 
       if (NULL != ptarget) {
-	freelog (LOG_DEBUG, "sabotage: random: targeted improvement: %d (%s)",
-		improvement_number(ptarget),
-		improvement_rule_name(ptarget));
+        log_debug("sabotage: random: targeted improvement: %d (%s)",
+                  improvement_number(ptarget),
+                  improvement_rule_name(ptarget));
       } else {
-	freelog (LOG_ERROR, "sabotage: random: targeted improvement error!");
+        log_error("sabotage: random: targeted improvement error!");
       }
     }
   } else {
@@ -949,20 +937,18 @@ void diplomat_sabotage(struct player *pplayer, struct unit *pdiplomat,
     if (city_has_building(pcity, pimprove)) {
       if (pimprove->sabotage > 0) {
 	ptarget = pimprove;
-	freelog (LOG_DEBUG, "sabotage: specified target improvement: %d (%s)",
-	       improvement,
-	       improvement_rule_name(pimprove));
+        log_debug("sabotage: specified target improvement: %d (%s)",
+                  improvement, improvement_rule_name(pimprove));
       } else {
         notify_player(pplayer, city_tile(pcity),
                       E_MY_DIPLOMAT_FAILED, ftc_server,
                       _("You cannot sabotage a %s!"),
                       improvement_name_translation(pimprove));
-	diplomat_charge_movement (pdiplomat, pcity->tile);
-	send_unit_info (pplayer, pdiplomat);
-	freelog (LOG_DEBUG, "sabotage: disallowed target improvement: %d (%s)",
-	       improvement,
-	       improvement_rule_name(pimprove));
-	return;
+        diplomat_charge_movement(pdiplomat, pcity->tile);
+        send_unit_info(pplayer, pdiplomat);
+        log_debug("sabotage: disallowed target improvement: %d (%s)",
+                  improvement, improvement_rule_name(pimprove));
+        return;
       }
     } else {
       notify_player(pplayer, city_tile(pcity),
@@ -971,11 +957,10 @@ void diplomat_sabotage(struct player *pplayer, struct unit *pdiplomat,
                     unit_name_translation(pdiplomat),
                     improvement_name_translation(pimprove),
                     city_link(pcity));
-      diplomat_charge_movement (pdiplomat, pcity->tile);
-      send_unit_info (pplayer, pdiplomat);
-      freelog (LOG_DEBUG, "sabotage: target improvement not found: %d (%s)",
-	       improvement,
-	       improvement_rule_name(pimprove));
+      diplomat_charge_movement(pdiplomat, pcity->tile);
+      send_unit_info(pplayer, pdiplomat);
+      log_debug("sabotage: target improvement not found: %d (%s)",
+                improvement, improvement_rule_name(pimprove));
       return;
     }
   }
@@ -1005,7 +990,7 @@ void diplomat_sabotage(struct player *pplayer, struct unit *pdiplomat,
                   prod,
                   city_link(pcity),
                   nation_plural_for_player(pplayer));
-    freelog (LOG_DEBUG, "sabotage: sabotaged production");
+    log_debug("sabotage: sabotaged production");
   } else {
     int vulnerability = ptarget->sabotage;
 
@@ -1019,7 +1004,7 @@ void diplomat_sabotage(struct player *pplayer, struct unit *pdiplomat,
     vulnerability -= (vulnerability
 		      * get_city_bonus(pcity, EFT_SPY_RESISTANT) / 100);
 
-    if (myrand(100) >= vulnerability) {
+    if (fc_rand(100) >= vulnerability) {
       /* Caught! */
       notify_player(pplayer, city_tile(pcity),
                     E_MY_DIPLOMAT_FAILED, ftc_server,
@@ -1033,8 +1018,8 @@ void diplomat_sabotage(struct player *pplayer, struct unit *pdiplomat,
                     unit_tile_link(pdiplomat),
                     improvement_name_translation(ptarget),
                     city_link(pcity));
-      wipe_unit(pdiplomat);
-      freelog(LOG_DEBUG, "sabotage: caught in capital or on city walls");
+      wipe_unit(pdiplomat, TRUE, cplayer);
+      log_debug("sabotage: caught in capital or on city walls");
       return;
     }
 
@@ -1051,9 +1036,9 @@ void diplomat_sabotage(struct player *pplayer, struct unit *pdiplomat,
                   nation_plural_for_player(pplayer),
                   improvement_name_translation(ptarget),
                   city_link(pcity));
-    freelog (LOG_DEBUG, "sabotage: sabotaged improvement: %d (%s)",
-	       improvement_number(ptarget),
-	       improvement_rule_name(ptarget));
+    log_debug("sabotage: sabotaged improvement: %d (%s)",
+              improvement_number(ptarget),
+              improvement_rule_name(ptarget));
 
     /* Do it. */
     building_lost(pcity, ptarget);
@@ -1063,7 +1048,8 @@ void diplomat_sabotage(struct player *pplayer, struct unit *pdiplomat,
   send_city_info(NULL, pcity);
 
   /* this may cause a diplomatic incident */
-  maybe_cause_incident(DIPLOMAT_SABOTAGE, pplayer, NULL, pcity);
+  maybe_cause_incident(DIPLOMAT_SABOTAGE, pplayer, cplayer,
+                       city_tile(pcity), city_link(pcity));
 
   /* Check if a spy survives her mission. Diplomats never do. */
   diplomat_escape(pplayer, pdiplomat, pcity);
@@ -1100,6 +1086,8 @@ static bool diplomat_success_vs_defender(struct unit *pattacker,
   if (unit_has_type_flag(pdefender, F_SUPERSPY)) {
     return FALSE;
   }
+
+  /* Add or remove 25% if spy flag. */
   if (unit_has_type_flag(pattacker, F_SPY)) {
     chance += 25;
   }
@@ -1107,20 +1095,23 @@ static bool diplomat_success_vs_defender(struct unit *pattacker,
     chance -= 25;
   }
 
-  chance += 15 * pattacker->veteran;
-  chance -= 15 * pdefender->veteran;
+  /* Add or remove up to 20% for veteran level. */
+  chance += 20 * pattacker->veteran / unit_type(pattacker)->veteran_levels;
+  chance -= 20 * pdefender->veteran / unit_type(pdefender)->veteran_levels;
 
   if (tile_city(pdefender_tile)) {
+    /* Reduce the chance of an attack by EFT_SPY_RESISTANCE percent. */
     chance -= chance * get_city_bonus(tile_city(pdefender_tile),
                                       EFT_SPY_RESISTANT) / 100;
   } else {
+    /* Reduce the chance of an attack if BF_DIPLOMAT_DEFENSE is active. */
     if (tile_has_base_flag_for_unit(pdefender_tile, unit_type(pdefender),
                                     BF_DIPLOMAT_DEFENSE)) {
       chance -= chance * 25 / 100; /* 25% penalty */
     }
   }
-  
-  return myrand(100) < chance;
+
+  return fc_rand(100) < chance;
 }
 
 /**************************************************************************
@@ -1219,8 +1210,13 @@ static bool diplomat_infiltrate_tile(struct player *pplayer,
         }
 
         pdiplomat->moves_left = MAX(0, pdiplomat->moves_left - SINGLE_MOVE);
+
+        /* Attacking unit became more experienced? */
+        if (maybe_make_veteran(pdiplomat)) {
+          notify_unit_experience(pdiplomat);
+        }
         send_unit_info(pplayer, pdiplomat);
-        wipe_unit(punit);
+        wipe_unit(punit, TRUE, pplayer);
         return FALSE;
       } else {
         /* Attacking Spy/Diplomat dies. */
@@ -1276,7 +1272,7 @@ static bool diplomat_infiltrate_tile(struct player *pplayer,
 	if (maybe_make_veteran(punit)) {
 	  notify_unit_experience(punit);
 	}
-	wipe_unit(pdiplomat);
+	wipe_unit(pdiplomat, TRUE, cplayer);
 	return FALSE;
       }
     }
@@ -1289,7 +1285,7 @@ static bool diplomat_infiltrate_tile(struct player *pplayer,
   This determines if a diplomat/spy survives and escapes.
   If "pcity" is NULL, assume action was in the field.
 
-  Spies have a game.info.diplchance specified chance of survival (better 
+  Spies have a game.server.diplchance specified chance of survival (better 
   if veteran):
     - Diplomats always die.
     - Escapes to home city.
@@ -1302,7 +1298,7 @@ static void diplomat_escape(struct player *pplayer, struct unit *pdiplomat,
   int escapechance;
   struct city *spyhome;
 
-  escapechance = game.info.diplchance + pdiplomat->veteran * 5;
+  escapechance = game.server.diplchance + pdiplomat->veteran * 5;
   
   if (pcity) {
     ptile = pcity->tile;
@@ -1311,11 +1307,13 @@ static void diplomat_escape(struct player *pplayer, struct unit *pdiplomat,
   }
   
   /* find closest city for escape target */
-  spyhome = find_closest_owned_city(unit_owner(pdiplomat), ptile, FALSE, NULL);
+  spyhome = find_closest_city(ptile, NULL, unit_owner(pdiplomat), FALSE,
+                              FALSE, FALSE, TRUE, FALSE);
 
   if (spyhome
       && unit_has_type_flag(pdiplomat, F_SPY)
-      && (myrand (100) < escapechance || unit_has_type_flag(pdiplomat, F_SUPERSPY))) {
+      && (unit_has_type_flag(pdiplomat, F_SUPERSPY)
+          || fc_rand (100) < escapechance)) {
     /* Attacking Spy/Diplomat survives. */
     notify_player(pplayer, ptile, E_MY_DIPLOMAT_ESCAPE, ftc_server,
                   _("Your %s has successfully completed"
@@ -1329,7 +1327,7 @@ static void diplomat_escape(struct player *pplayer, struct unit *pdiplomat,
     /* being teleported costs all movement */
     if (!teleport_unit_to_city (pdiplomat, spyhome, -1, FALSE)) {
       send_unit_info (pplayer, pdiplomat);
-      freelog(LOG_ERROR, "Bug in diplomat_escape: Spy can't teleport.");
+      log_error("Bug in diplomat_escape: Spy can't teleport.");
       return;
     }
 
@@ -1349,28 +1347,19 @@ static void diplomat_escape(struct player *pplayer, struct unit *pdiplomat,
     }
   }
 
-  wipe_unit(pdiplomat);
+  wipe_unit(pdiplomat, FALSE, NULL);
 }
 
-/**************************************************************************
-...
-**************************************************************************/
-static void maybe_cause_incident(enum diplomat_actions action, struct player *offender,
- 				 struct unit *victim_unit, struct city *victim_city)
+/****************************************************************************
+  After an action of a diplomat, maybe it will cause a diplomatic incident
+  with the victim.
+****************************************************************************/
+static void maybe_cause_incident(enum diplomat_actions action,
+                                 struct player *offender,
+                                 struct player *victim_player,
+                                 struct tile *victim_tile,
+                                 const char *victim_link)
 {
-  struct player *victim_player = 0;
-  struct tile *victim_tile = NULL;
-
-  if (victim_city) {
-    victim_tile = victim_city->tile;
-    victim_player = city_owner(victim_city);
-  } else if (victim_unit) {
-    victim_tile = victim_unit->tile;
-    victim_player = unit_owner(victim_unit);
-  } else {
-    die("No victim in call to maybe_cause_incident()");
-  }
-
   if (!pplayers_at_war(offender, victim_player)) {
     switch (action) {
     case DIPLOMAT_BRIBE:
@@ -1378,38 +1367,32 @@ static void maybe_cause_incident(enum diplomat_actions action, struct player *of
                     E_DIPLOMATIC_INCIDENT, ftc_server,
                     _("You have caused an incident while bribing "
                       "the %s %s."),
-                    nation_adjective_for_player(victim_player),
-                    unit_link(victim_unit));
+                    nation_adjective_for_player(victim_player), victim_link);
       notify_player(victim_player, victim_tile,
                     E_DIPLOMATIC_INCIDENT, ftc_server,
                     _("%s has caused an incident while bribing your %s."),
-                    player_name(offender),
-                    unit_link(victim_unit));
+                    player_name(offender), victim_link);
       break;
     case DIPLOMAT_STEAL:
       notify_player(offender, victim_tile,
                     E_DIPLOMATIC_INCIDENT, ftc_server,
                     _("You have caused an incident while attempting "
-                      "to steal tech from %s."),
-                    player_name(victim_player));
+                      "to steal tech from %s."), player_name(victim_player));
       notify_player(victim_player, victim_tile,
                     E_DIPLOMATIC_INCIDENT, ftc_server,
                     _("%s has caused an incident while attempting "
-                      "to steal tech from you."),
-                    player_name(offender));
+                      "to steal tech from you."), player_name(offender));
       break;
     case DIPLOMAT_INCITE:
       notify_player(offender, victim_tile,
                     E_DIPLOMATIC_INCIDENT, ftc_server,
                     _("You have caused an incident while inciting a "
-                      "revolt in %s."),
-                    city_link(victim_city));
+                      "revolt in %s."), victim_link);
       notify_player(victim_player, victim_tile,
                     E_DIPLOMATIC_INCIDENT, ftc_server,
-                    _("%s have caused an incident while inciting a "
+                    _("The %s have caused an incident while inciting a "
                       "revolt in %s."),
-                    nation_plural_for_player(offender),
-                    city_link(victim_city));
+                    nation_plural_for_player(offender), victim_link);
       break;
     case DIPLOMAT_MOVE:
     case DIPLOMAT_EMBASSY:
@@ -1420,13 +1403,13 @@ static void maybe_cause_incident(enum diplomat_actions action, struct player *of
     case SPY_SABOTAGE_UNIT:
     case DIPLOMAT_SABOTAGE:
       /* You can only do these when you are at war, so we should never
- 	 get inside this "if" */
-      die("Bug in maybe_cause_incident()");
+       * get inside this "if" */
+      fc_assert_ret(FALSE);
     }
-    victim_player->diplstates[player_index(offender)].has_reason_to_cancel = 2;
+    player_diplstate_get(victim_player, offender)->has_reason_to_cancel = 2;
     call_incident(INCIDENT_DIPLOMAT, offender, victim_player);
-    send_player_info(offender, NULL);
-    send_player_info(victim_player, NULL);
+    send_player_all_c(offender, NULL);
+    send_player_all_c(victim_player, NULL);
   }
 
   return;
@@ -1447,8 +1430,8 @@ int unit_bribe_cost(struct unit *punit)
   int dist;
   int default_hp = unit_type(punit)->hp;
 
-  cost = unit_owner(punit)->economic.gold + game.info.base_bribe_cost;
-  capital = find_palace(unit_owner(punit));
+  cost = unit_owner(punit)->economic.gold + game.server.base_bribe_cost;
+  capital = player_palace(unit_owner(punit));
   if (capital) {
     int tmp = map_distance(capital->tile, punit->tile);
     dist=MIN(32, tmp);
