@@ -1018,11 +1018,7 @@ static void make_rivers(void)
   create_placed_map(); /* needed bu rand_map_characteristic */
   set_all_ocean_tiles_placed();
 
-  rivermap.blocked.vec = NULL;
-  rivermap.blocked.bits = 0;
   dbv_init(&rivermap.blocked, MAP_INDEX_SIZE);
-  rivermap.ok.vec = NULL;
-  rivermap.ok.bits = 0;
   dbv_init(&rivermap.ok, MAP_INDEX_SIZE);
 
   /* The main loop in this function. */
@@ -1139,7 +1135,7 @@ static void make_land(void)
   fc_assert_exit_msg(NULL != land_fill,
                      "No land terrain type could be found for the purpose "
                      "of temporarily filling in land tiles during map "
-                     "generation. This could be an error in freeciv, or a "
+                     "generation. This could be an error in Freeciv, or a "
                      "mistake in the terrain.ruleset file. Please make sure "
                      "there is at least one land terrain type in the "
                      "ruleset, or use a different map generator. If this "
