@@ -1029,9 +1029,6 @@ void player_map_free(struct player *pplayer)
   pplayer->server.private_map = NULL;
 
   dbv_free(&pplayer->tile_known);
-
-  /* Recalculate borders. */
-  map_calculate_borders();
 }
 
 /***************************************************************
@@ -1502,7 +1499,7 @@ static void check_units_single_tile(struct tile *ptile)
                       E_UNIT_LOST_MISC, ftc_server,
                       _("Disbanded your %s due to changing terrain."),
                       unit_tile_link(punit));
-        wipe_unit(punit);
+        wipe_unit(punit, TRUE, NULL);
       }
     }
   } unit_list_iterate_safe_end;
@@ -1699,6 +1696,11 @@ static void map_claim_ownership_full(struct tile *ptile,
 
   tile_set_owner(ptile, powner, psource);
 
+  /* Needed only when foggedborders enabled, but we do it unconditionally
+   * in case foggedborders ever gets enabled later. Better to have correct
+   * information in player map just in case. */
+  update_tile_knowledge(ptile);
+
   if (ploser != powner) {
     if (S_S_RUNNING == server_state() && game.info.happyborders > 0) {
       map_unit_homecity_enqueue(ptile);
@@ -1754,11 +1756,12 @@ void map_claim_border(struct tile *ptile, struct player *owner)
       continue;
     }
 
-    if (!map_is_known(dtile, owner) && BORDERS_EXPAND != game.info.borders) {
+    if (!map_is_known(dtile, owner) && game.info.borders < BORDERS_EXPAND) {
       continue;
     }
 
-    if (NULL != dclaimer && dclaimer != ptile) {
+    /* Always claim source itself (distance, dr, to it 0) */
+    if (dr != 0 && NULL != dclaimer && dclaimer != ptile) {
       struct city *ccity = tile_city(dclaimer);
       int strength_old, strength_new;
 
@@ -1806,6 +1809,11 @@ void map_calculate_borders(void)
     return;
   }
 
+  if (map.tiles == NULL) {
+    /* Map not yet initialized */
+    return;
+  }
+
   log_verbose("map_calculate_borders()");
 
   whole_map_iterate(ptile) {
