@@ -142,7 +142,7 @@ void kill_player(struct player *pplayer)
 
   /* Remove all units that are still ours */
   unit_list_iterate_safe(pplayer->units, punit) {
-    wipe_unit(punit);
+    wipe_unit(punit, FALSE, NULL);
   } unit_list_iterate_safe_end;
 
   /* Destroy any remaining cities */
@@ -166,6 +166,21 @@ void kill_player(struct player *pplayer)
   send_player_info_c(pplayer, game.est_connections);
 }
 
+/**************************************************************************
+  Return player maxrate in legal range.
+**************************************************************************/
+static int get_player_maxrate(struct player *pplayer)
+{
+  int maxrate = get_player_bonus(pplayer, EFT_MAX_RATES);
+
+  if (maxrate == 0) {
+    return 100; /* effects not initialized yet */
+  }
+
+  /* 34 + 33 + 33 = 100 */
+  return CLIP(34, maxrate, 100);
+}
+
 /**************************************************************************
   Handle a client or AI request to change the tax/luxury/science rates.
   This function does full sanity checking.
@@ -182,7 +197,7 @@ void handle_player_rates(struct player *pplayer,
                   _("Cannot change rates before game start."));
     return;
   }
-	
+
   if (tax + luxury + science != 100) {
     return;
   }
@@ -190,7 +205,7 @@ void handle_player_rates(struct player *pplayer,
       || science > 100) {
     return;
   }
-  maxrate = get_player_bonus(pplayer, EFT_MAX_RATES);
+  maxrate = get_player_maxrate(pplayer);
   if (tax > maxrate || luxury > maxrate || science > maxrate) {
     const char *rtype;
 
@@ -245,7 +260,7 @@ static void finish_revolution(struct player *pplayer)
 
   if (!pplayer->ai_controlled) {
     /* Keep luxuries if we have any.  Try to max out science. -GJW */
-    int max = get_player_bonus(pplayer, EFT_MAX_RATES);
+    int max = get_player_maxrate(pplayer);
 
     /* only change rates if one exceeds the maximal rate */
     if (pplayer->economic.science > max || pplayer->economic.tax > max
@@ -455,6 +470,55 @@ void update_players_after_alliance_breakup(struct player* pplayer,
    resolve_unit_stacks(pplayer, pplayer2, TRUE);
 }
 
+/**************************************************************************
+  If there's any units of new_owner on tile, they claim bases.
+**************************************************************************/
+static void maybe_claim_base(struct tile *ptile, struct player *new_owner,
+                             struct player *old_owner)
+{
+  bool claim = FALSE;
+
+  if (BORDERS_DISABLED == game.info.borders) {
+    return;
+  }
+
+  unit_list_iterate(ptile->units, punit) {
+    if (unit_owner(punit) == new_owner
+        && tile_has_claimable_base(ptile, unit_type(punit))) {
+      claim = TRUE;
+      break;
+    }
+  } unit_list_iterate_end;
+
+  if (claim) {
+    map_claim_ownership(ptile, new_owner, ptile);
+
+    /* Clear borders from old owner. New owner may not know all those
+     * tiles and thus does not claim them when borders mode is less
+     * than EXPAND. */
+    map_clear_border(ptile);
+    map_claim_border(ptile, new_owner);
+    city_thaw_workers_queue();
+    city_refresh_queue_processing();
+  }
+}
+
+/**************************************************************************
+  Two players enter war.
+**************************************************************************/
+void enter_war(struct player *pplayer, struct player *pplayer2)
+{
+  /* Claim bases where units are already standing */
+  whole_map_iterate(ptile) {
+    struct player *old_owner = tile_owner(ptile);
+
+    if (old_owner == pplayer2) {
+      maybe_claim_base(ptile, pplayer, old_owner);
+    } else if (old_owner == pplayer) {
+      maybe_claim_base(ptile, pplayer2, old_owner);
+    }
+  } whole_map_iterate_end;
+}
 
 /**************************************************************************
   Handles a player cancelling a "pact" with another player.
@@ -562,6 +626,8 @@ void handle_diplomacy_cancel_pact(struct player *pplayer,
   }
   if (new_type == DS_WAR) {
     call_incident(INCIDENT_WAR, pplayer, pplayer2);
+
+    enter_war(pplayer, pplayer2);
   }
   ds_plrplr2->has_reason_to_cancel = 0;
 
@@ -1197,6 +1263,9 @@ void server_remove_player(struct player *pplayer)
   send_updated_vote_totals(NULL);
   /* must be called after the player was destroyed */
   send_player_remove_info_c(pslot, NULL);
+
+  /* Recalculate borders. */
+  map_calculate_borders();
 }
 
 /**************************************************************************
@@ -1221,10 +1290,7 @@ struct player_economic player_limit_to_max_rates(struct player *pplayer)
 
   economic = pplayer->economic;
 
-  maxrate = get_player_bonus(pplayer, EFT_MAX_RATES);
-  if (maxrate == 0) {
-    maxrate = 100; /* effects not initialized yet */
-  }
+  maxrate = get_player_maxrate(pplayer);
 
   surplus = 0;
   if (economic.luxury > maxrate) {
