@@ -389,31 +389,6 @@ bool check_for_game_over(void)
         player_list_append(winner_list, pplayer);
       } players_iterate_end;
 
-      if (NULL != winner_list) {
-        /* Now ensure a non allied has conceded the game. */
-        bool found = FALSE;
-
-        players_iterate(pplayer) {
-          if (is_barbarian(pplayer)
-              || PLAYER_IS_CANDIDATE(pplayer)) {
-            continue;
-          }
-
-          player_list_iterate(winner_list, aplayer) {
-            if (!pplayers_allied(aplayer, pplayer)) {
-              found = TRUE;
-              break;
-            }
-          } player_list_iterate_end;
-        } players_iterate_end;
-
-        if (!found) {
-          /* Seems all players are allied. */
-          player_list_destroy(winner_list);
-          winner_list = NULL;
-        }
-      }
-
       if (NULL != winner_list) {
         bool first = TRUE;
 
@@ -643,7 +618,7 @@ static void remove_illegal_armistice_units(struct player *plr1,
                       "your peace treaty with the %s."),
                     unit_tile_link(punit),
                     nation_plural_for_player(plr2));
-      wipe_unit(punit);
+      wipe_unit(punit, TRUE, NULL);
     }
   } unit_list_iterate_safe_end;
   unit_list_iterate_safe(plr2->units, punit) {
@@ -654,7 +629,7 @@ static void remove_illegal_armistice_units(struct player *plr1,
                       "your peace treaty with the %s."),
                     unit_tile_link(punit),
                     nation_plural_for_player(plr1));
-      wipe_unit(punit);
+      wipe_unit(punit, TRUE, NULL);
     }
   } unit_list_iterate_safe_end;
 }
@@ -711,6 +686,9 @@ static void update_diplomatics(void)
           state2->type = DS_WAR;
           state->turns_left = 0;
           state2->turns_left = 0;
+
+          enter_war(plr1, plr2);
+
           city_map_update_all_cities_for_player(plr1);
           city_map_update_all_cities_for_player(plr2);
           sync_cities();
@@ -840,7 +818,8 @@ static void begin_turn(bool is_new_turn)
    * player for ranking purposes. */
   players_iterate(pplayer) {
     if (strcmp(pplayer->ranked_username, ANON_USER_NAME) == 0
-        && pplayer->user_turns++ > TURNS_NEEDED_TO_RANK) {
+        && pplayer->user_turns++ > TURNS_NEEDED_TO_RANK
+	&& pplayer->is_alive) {
       sz_strlcpy(pplayer->ranked_username, pplayer->username);
     }
   } players_iterate_end;
@@ -1159,9 +1138,22 @@ void save_game(const char *orig_filename, const char *save_reason,
     if ('\0' == *dot) {
       /* Only dots in this file name, consider it as empty. */
       filename[0] = '\0';
-    } else if ((dot = strchr(dot, '.'))) {
-      /* Strikes extension. */
-      *dot = '\0';
+    } else {
+      char *end_dot;
+      char *strip_extensions[] = { ".sav", ".gz", ".bz2", NULL };
+      bool stripped = TRUE;
+
+      while ((end_dot = strrchr(dot, '.')) && stripped) {
+	int i;
+        stripped = FALSE;
+
+	for (i = 0; strip_extensions[i] != NULL && !stripped; i++) {
+          if (!strcmp(end_dot, strip_extensions[i])) {
+            *end_dot = '\0';
+            stripped = TRUE;
+          }
+        }
+      }
     }
   }
 
@@ -1864,6 +1856,16 @@ void aifill(int amount)
 ****************************************************************************/
 static void player_set_nation_full(struct player *pplayer,
                                    struct nation_type *pnation)
+{
+  /* Don't change the name of a created player. */
+  player_nation_defaults(pplayer, pnation, !pplayer->was_created);
+}
+
+/****************************************************************************
+  Set nation for player with nation default values.
+****************************************************************************/
+void player_nation_defaults(struct player *pplayer, struct nation_type *pnation,
+                            bool set_name)
 {
   struct nation_leader *pleader;
 
@@ -1871,17 +1873,13 @@ static void player_set_nation_full(struct player *pplayer,
   player_set_nation(pplayer, pnation);
   fc_assert(pnation == pplayer->nation);
 
-  pplayer->city_style =
-      city_style_of_nation(nation_of_player(pplayer));
+  pplayer->city_style = city_style_of_nation(nation_of_player(pplayer));
 
-  /* Don't change the name of a created player. */
-  if (!pplayer->was_created) {
-    server_player_set_name(pplayer, pick_random_player_name
-                           (nation_of_player(pplayer)));
+  if (set_name) {
+    server_player_set_name(pplayer, pick_random_player_name(pnation));
   }
 
-  if ((pleader = nation_leader_by_name(nation_of_player(pplayer),
-                                       player_name(pplayer)))) {
+  if ((pleader = nation_leader_by_name(pnation, player_name(pplayer)))) {
     pplayer->is_male = nation_leader_is_male(pleader);
   } else {
     pplayer->is_male = (fc_rand(2) == 1);
@@ -2375,7 +2373,15 @@ static void srv_ready(void)
   if (map_is_empty()
       || (MAPGEN_SCENARIO == map.server.generator
           && game.info.is_new_game)) {
-    struct unit_type *utype = crole_to_unit_type(game.server.start_units[0], NULL);
+    struct unit_type *utype = NULL;
+    int sucount = strlen(game.server.start_units);
+    int i;
+
+    for (i = 0; utype == NULL && i < sucount; i++) {
+      utype = crole_to_unit_type(game.server.start_units[i], NULL);
+    }
+
+    fc_assert(utype != NULL);
 
     map_fractal_generate(TRUE, utype);
     game_map_init();
