@@ -543,7 +543,7 @@ void handle_unit_disband(struct player *pplayer, int unit_id)
     send_city_info(city_owner(pcity), pcity);
   }
 
-  wipe_unit(punit);
+  wipe_unit(punit, FALSE, NULL);
 }
 
 /**************************************************************************
@@ -674,7 +674,7 @@ static void city_add_unit(struct player *pplayer, struct unit *punit)
                 _("%s added to aid %s in growing."),
                 unit_tile_link(punit),
                 city_link(pcity));
-  wipe_unit(punit);
+  wipe_unit(punit, FALSE, NULL);
   send_city_info(NULL, pcity);
 }
 
@@ -705,7 +705,7 @@ static void city_build(struct player *pplayer, struct unit *punit,
 
     city_change_size(pcity, size);
   }
-  wipe_unit(punit);
+  wipe_unit(punit, FALSE, NULL);
 }
 
 /**************************************************************************
@@ -1039,15 +1039,13 @@ static void unit_attack_handling(struct unit *punit, struct unit *pdefender)
       notify_player(city_owner(pcity), def_tile, E_UNIT_WIN, ftc_server,
                     _("The nuclear attack on %s was avoided by"
                       " your SDI defense."), city_link(pcity));
-      pplayer->score.units_lost++;
-      city_owner(pcity)->score.units_killed++;
-      wipe_unit(punit);
+      wipe_unit(punit, TRUE, city_owner(pcity));
       return;
     } 
 
     dlsend_packet_nuke_tile_info(game.est_connections, tile_index(def_tile));
 
-    wipe_unit(punit);
+    wipe_unit(punit, FALSE, NULL);
     do_nuclear_explosion(pplayer, def_tile);
     return;
   }
@@ -1130,9 +1128,7 @@ static void unit_attack_handling(struct unit *punit, struct unit *pdefender)
                   loser_link,
                   nation_adjective_for_player(unit_owner(pwinner)),
                   winner_link);
-    unit_owner(ploser)->score.units_lost++;
-    unit_owner(pwinner)->score.units_killed++;
-    wipe_unit(ploser);
+    wipe_unit(ploser, TRUE, unit_owner(pwinner));
   } else {
     /* The defender lost, the attacker punit lives! */
     int winner_id = pwinner->id;
@@ -1148,7 +1144,7 @@ static void unit_attack_handling(struct unit *punit, struct unit *pdefender)
               vet && !uclass_has_flag(unit_class(punit), UCF_MISSILE));
     if (unit_alive(winner_id)) {
       if (uclass_has_flag(unit_class(pwinner), UCF_MISSILE)) {
-        wipe_unit(pwinner);
+        wipe_unit(pwinner, FALSE, NULL);
         return;
       }
     } else {
@@ -1315,7 +1311,7 @@ bool unit_move_handling(struct unit *punit, struct tile *pdesttile,
     struct unit *target = is_non_allied_unit_tile(pdesttile, pplayer);
 
     if (target || is_non_allied_city_tile(pdesttile, pplayer)
-        || !move_diplomat_city) {
+        || (is_allied_city_tile(pdesttile, pplayer) && !move_diplomat_city)) {
       if (is_diplomat_action_available(punit, DIPLOMAT_ANY_ACTION,
 				       pdesttile)) {
 	int target_id = 0;
@@ -1549,7 +1545,7 @@ void handle_unit_help_build_wonder(struct player *pplayer, int unit_id)
                 city_link(pcity_dest), 
                 abs(build_points_left(pcity_dest)));
 
-  wipe_unit(punit);
+  wipe_unit(punit, FALSE, NULL);
   send_player_info_c(pplayer, pplayer->connections);
   send_city_info(pplayer, pcity_dest);
   conn_list_do_unbuffer(pplayer->connections);
@@ -1694,7 +1690,7 @@ static bool base_handle_unit_establish_trade(struct player *pplayer, int unit_id
                 homecity_link,
                 destcity_link,
                 revenue);
-  wipe_unit(punit);
+  wipe_unit(punit, FALSE, NULL);
   pplayer->economic.gold += revenue;
   /* add bulbs and check for finished research */
   update_bulbs(pplayer, revenue, TRUE);
