@@ -85,6 +85,10 @@
 #  include <strings.h>
 #endif
 
+#ifdef HAVE_LIBZ
+#include <zlib.h>
+#endif
+
 /* utility */
 #include "fciconv.h"
 #include "fcintl.h"
@@ -283,10 +287,13 @@ int fc_strcoll(const char *str0, const char *str1)
 ****************************************************************************/
 int fc_stricoll(const char *str0, const char *str1)
 {
-#if defined(ENABLE_NLS) && defined(HAVE_STRICOLL)
-  return stricoll(str0, str1);
-#elif defined(ENABLE_NLS) && defined(HAVE__STRICOLL)
+  /* We prefer _stricoll() over stricoll() since
+   * latter is not declared in MinGW headers causing compiler
+   * warning, preventing -Werror builds. */
+#if defined(ENABLE_NLS) && defined(HAVE__STRICOLL)
   return _stricoll(str0, str1);
+#elif defined(ENABLE_NLS) && defined(HAVE_STRICOLL)
+  return stricoll(str0, str1);
 #elif defined(ENABLE_NLS) && defined(HAVE_STRCASECOLL)
   return strcasecoll(str0, str1);
 #else
@@ -317,10 +324,10 @@ FILE *fc_fopen(const char *filename, const char *opentype)
   encoding on Windows.
 *****************************************************************/
 #ifdef HAVE_LIBZ
-FILE *fc_gzopen(const char *filename, const char *opentype)
+gzFile fc_gzopen(const char *filename, const char *opentype)
 {
 #ifdef WIN32_NATIVE
-	FILE *result;
+	gzFile result;
 	char *filename_in_local_encoding =
 	     internal_to_local_string_malloc(filename);
 	result = gzopen(filename_in_local_encoding, opentype);
@@ -678,7 +685,9 @@ size_t fc_strlcat(char *dest, const char *src, size_t n)
 #define VSNP_BUF_SIZE (64*1024)
 int fc_vsnprintf(char *str, size_t n, const char *format, va_list ap)
 {
+#ifdef HAVE_WORKING_VSNPRINTF
   int r;
+#endif
 
   /* This may be overzealous, but I suspect any triggering of these to
    * be bugs.  */
@@ -692,8 +701,9 @@ int fc_vsnprintf(char *str, size_t n, const char *format, va_list ap)
   str[n - 1] = 0;
 
   /* Convert C99 return value to C89.  */
-  if (r >= n)
+  if (r >= n) {
     return -1;
+  }
 
   return r;
 #else
@@ -714,10 +724,10 @@ int fc_vsnprintf(char *str, size_t n, const char *format, va_list ap)
       }
     }
 #ifdef HAVE_VSNPRINTF
-    r = vsnprintf(buf, n, format, ap);
+    vsnprintf(buf, n, format, ap);
 #else
-    r = vsprintf(buf, format, ap);
-#endif
+    vsprintf(buf, format, ap);
+#endif /* HAVE_VSNPRINTF */
     buf[VSNP_BUF_SIZE - 1] = '\0';
     len = strlen(buf);
 
