@@ -44,7 +44,6 @@
 #include <config.h>
 #endif
 
-#include <assert.h>
 #include <time.h>
 
 #ifdef HAVE_GETTIMEOFDAY
@@ -111,7 +110,7 @@ static void report_clock_failed(struct timer *t)
   static bool first = TRUE;
 
   if (first) {
-    freelog(LOG_TEST, "clock() returned -1, ignoring timer");
+    log_test("clock() returned -1, ignoring timer");
     first = FALSE;
   }
   t->use = TIMER_IGNORE;
@@ -127,7 +126,7 @@ static void report_gettimeofday_failed(struct timer *t)
   static bool first = TRUE;
 
   if (first) {
-    freelog(LOG_TEST, "gettimeofday() returned -1, ignoring timer");
+    log_test("gettimeofday() returned -1, ignoring timer");
     first = FALSE;
   }
   t->use = TIMER_IGNORE;
@@ -142,7 +141,7 @@ static void report_time_failed(struct timer *t)
   static bool first = TRUE;
 
   if (first) {
-    freelog(LOG_TEST, "time() returned -1, ignoring timer");
+    log_test("time() returned -1, ignoring timer");
     first = FALSE;
   }
   t->use = TIMER_IGNORE;
@@ -178,7 +177,7 @@ struct timer *new_timer_start(enum timer_timetype type, enum timer_use use)
      static struct timer *t = NULL; 
      t = renew_timer_start(t, TIMER_CPU, TIMER_USE);
      ... stuff ...
-     freelog(LOG_VERBOSE, "That took %g seconds.", read_timer_seconds(t));
+     log_verbose("That took %g seconds.", read_timer_seconds(t));
      ... never free t ...
   }
 ***********************************************************************/
@@ -234,7 +233,7 @@ bool timer_in_use(struct timer *t)
 ***********************************************************************/
 void clear_timer(struct timer *t)
 {
-  assert(t != NULL);
+  fc_assert_ret(NULL != t);
   t->state = TIMER_STOPPED;
   t->sec = 0.0;
   t->usec = 0;
@@ -246,13 +245,13 @@ void clear_timer(struct timer *t)
 ***********************************************************************/
 void start_timer(struct timer *t)
 {
-  assert(t != NULL);
+  fc_assert_ret(NULL != t);
 
   if (t->use == TIMER_IGNORE) {
     return;
   }
   if (t->state == TIMER_STARTED) {
-    freelog(LOG_ERROR, "tried to start already started timer");
+    log_error("tried to start already started timer");
     return;
   }
   if (t->type == TIMER_CPU) {
@@ -298,13 +297,13 @@ void clear_timer_start(struct timer *t)
 ***********************************************************************/
 void stop_timer(struct timer *t)
 {
-  assert(t != NULL);
-  
+  fc_assert_ret(NULL != t);
+
   if (t->use == TIMER_IGNORE) {
     return;
   }
   if (t->state == TIMER_STOPPED) {
-    freelog(LOG_ERROR, "tried to stop already stopped timer");
+    log_error("tried to stop already stopped timer");
     return;
   }
   if (t->type == TIMER_CPU) {
@@ -369,8 +368,8 @@ void stop_timer(struct timer *t)
 ***********************************************************************/
 double read_timer_seconds(struct timer *t)
 {
-  assert(t != NULL);
-  
+  fc_assert_ret_val(NULL != t, -1.0);
+
   if (t->use == TIMER_IGNORE) {
     return 0.0;
   }
@@ -381,24 +380,6 @@ double read_timer_seconds(struct timer *t)
   return t->sec + t->usec / (double)N_USEC_PER_SEC;
 }
 
-/********************************************************************** 
-  Read the timer, then free it.
-  This is intended to be useful for a simple one-off timing, eg:
-  {
-      struct timer *t = new_timer_start();
-      ...do stuff...
-      freelog(LOG_TEST, "That took %g seconds", read_timer_seconds_free(t));
-  }
-  (BUT: make sure the _free call really happens!
-  eg, freelog(LOG_DEBUG,...) might not actually evaluate its args.)
-***********************************************************************/
-double read_timer_seconds_free(struct timer *t)
-{
-  double val = read_timer_seconds(t);
-  free_timer(t);
-  return val;
-}
-
 /********************************************************************** 
   Sleeps until the given number of microseconds have elapsed since the
   timer was started.  Leaves the timer running.
@@ -413,7 +394,7 @@ void usleep_since_timer_start(struct timer *t, long usec)
   long elapsed_usec;
   long wait_usec;
 
-  assert(t != NULL);
+  fc_assert_ret(NULL != t);
 
   ret = gettimeofday(&tv_now, NULL);
 
@@ -421,7 +402,7 @@ void usleep_since_timer_start(struct timer *t, long usec)
       (t->type != TIMER_USER) ||
       (t->use != TIMER_ACTIVE) ||
       (t->state != TIMER_STARTED)) {
-    myusleep(usec);
+    fc_usleep(usec);
     return;
   }
 
@@ -431,7 +412,7 @@ void usleep_since_timer_start(struct timer *t, long usec)
   wait_usec = usec - elapsed_usec;
 
   if (wait_usec > 0)
-    myusleep(wait_usec);
+    fc_usleep(wait_usec);
 #elif HAVE_FTIME
   struct timeb now;
   long elapsed_usec, wait_usec;
@@ -443,10 +424,10 @@ void usleep_since_timer_start(struct timer *t, long usec)
   wait_usec = usec - elapsed_usec;
 
   if (wait_usec > 0) {
-    myusleep(wait_usec);
+    fc_usleep(wait_usec);
   }
 #else
-  myusleep(usec);
+  fc_usleep(usec);
 #endif
 }
 
